@@ -17,15 +17,15 @@
  *
  *
  *
- *  $Id: Effects.h,v 1.1 2002/03/25 15:42:29 adnans Exp $
+ *  $Id: Effects.h,v 1.2 2002/06/05 17:56:09 adnans Exp $
  *
 */ 
 
 #ifndef __Effects_h__
 #define __Effects_h__
 
-#define DELAY_BUF_SIZE        ((44100 * 2 * 2) * 1)
-#define MAX_CHUNK      32768
+#define DELAY_BUF_SIZE        ((44100 * 2 * 2) * 2)
+#define MAX_CHUNK      (32768 * 2)
 
 extern "C" {
 
