@@ -1,5 +1,5 @@
 /*  CorePlayer.cpp - Core player object, most of the hacking is done here!  
- *	Copyright (C) 1998-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *	Copyright (C) 1998-2003 Andy Lo A Foe <andy@alsaplayer.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,21 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: CorePlayer.cpp,v 1.19 2002/04/04 11:48:15 adnans Exp $
+ *  $Id: CorePlayer.cpp,v 1.65 2003/04/18 23:44:28 adnans Exp $
  *
 */ 
 
 
 /*
 	Issues:
-		- None at the moment
+		- Things need to be cleaned up!
 
 */
 
+#include "AlsaPlayer.h"
 #include "CorePlayer.h"
-#include <stdio.h>
-#include <string.h>
-#include <assert.h>
+#include <cstdio>
+#include <cstring>
+#include <cassert>
 #include <unistd.h>
 #include <dirent.h>
 #include <sys/stat.h>
@@ -44,18 +45,16 @@
 #include <dlfcn.h>
 #include <sched.h>
 #include <pthread.h>
-#include <math.h>
+#include <cmath>
 #include "Effects.h"
 #include "utilities.h"
 #include "alsaplayer_error.h"
 
-#define MAX_FRAGS	16
-#define LOW_FRAGS	4	
-
 extern void exit_sighandler(int);
 static char addon_dir[1024];
 
 
+#if 0	// unused; Frank
 bool surround_func(void *arg, void *data, int size)
 {
 	int amount = 50;
@@ -68,23 +67,72 @@ bool surround_func(void *arg, void *data, int size)
 	}
 	return true;
 }
+#endif
 
 
+int CorePlayer::plugin_count = 0;
+int CorePlayer::plugins_loaded = 0;
+pthread_mutex_t CorePlayer::plugins_mutex = PTHREAD_MUTEX_INITIALIZER;
+input_plugin CorePlayer::plugins[MAX_INPUT_PLUGINS];
+
 void CorePlayer::Lock()
 {
-	//alsaplayer_error("Getting player lock...");
-	pthread_mutex_lock(&player_mutex);
-	//alsaplayer_error("Got player lock...");
+	if (pthread_mutex_lock(&player_mutex))
+		alsaplayer_error("FIRE!!!!!");
 }
 
 
 void CorePlayer::Unlock()
 {
 	pthread_mutex_unlock(&player_mutex);
-	//alsaplayer_error("Released player lock...");
 }
 
 
+void CorePlayer::LockNotifiers()
+{
+	pthread_mutex_lock(&notifier_mutex);
+}
+
+
+void CorePlayer::UnlockNotifiers()
+{
+	 pthread_mutex_unlock(&notifier_mutex);
+}
+
+
+void CorePlayer::RegisterNotifier(coreplayer_notifier *core_notif, void *data)
+{
+	if (!core_notif) {
+		alsaplayer_error("Notifier is NULL");
+		return;
+	}	
+	if (data)
+		core_notif->data = data;
+	if (core_notif->speed_changed)
+		core_notif->speed_changed(core_notif->data, GetSpeed());
+	if (core_notif->pan_changed)
+		core_notif->pan_changed(core_notif->data, GetPan());
+	if (core_notif->volume_changed)
+		core_notif->volume_changed(core_notif->data, GetVolume());
+	if (core_notif->position_notify)
+		core_notif->position_notify(core_notif->data, GetPosition());
+	LockNotifiers();
+	notifiers.insert(core_notif);
+	UnlockNotifiers();
+}
+
+
+void CorePlayer::UnRegisterNotifier(coreplayer_notifier *core_notif)
+{
+	if (!core_notif)
+		return;
+	LockNotifiers();
+	notifiers.erase(notifiers.find(core_notif));
+	UnlockNotifiers();
+}
+
+
+
 void CorePlayer::load_input_addons()
 {
 	char path[1024];
@@ -96,76 +144,84 @@ void CorePlayer::load_input_addons()
 
 	sprintf(path, "%s/input", addon_dir);
 
-	dir = opendir(path);
+	memset(plugins, 0, sizeof(plugins));
 
-	if (dir) {
-		while ((entry = readdir(dir)) != NULL) {
-			if (strcmp(entry->d_name, ".") == 0 ||
-			    strcmp(entry->d_name, "..") == 0) {
-					continue;
-			}
-			sprintf(path, "%s/input/%s", addon_dir, entry->d_name);
-			if (stat(path, &buf)) continue;
-			if (S_ISREG(buf.st_mode)) {
-				void *handle;
+	if ((dir = opendir(path)) == NULL)
+		return;
 
-				char *ext = strrchr(path, '.');
+	while ((entry = readdir(dir)) != NULL) {
+		if (strcmp(entry->d_name, ".") == 0 ||
+		    strcmp(entry->d_name, "..") == 0) {
+				continue;
+		}
+		sprintf(path, "%s/input/%s", addon_dir, entry->d_name);
+		if (stat(path, &buf)) continue;
+		if (!S_ISREG(buf.st_mode)) continue;
 
-				if (!ext)
-					continue;
+		void *handle;
+
+		char *ext = strrchr(path, '.');
+		if (!ext)
+			continue;
 				
-				ext++;
+		ext++;
+
+		if (strcasecmp(ext, "so"))
+			continue;
+
+		if ((handle = dlopen(path, RTLD_NOW |RTLD_GLOBAL)) == NULL) {
+			alsaplayer_error("%s", dlerror());
+			continue;
+		}
 
-				if (strcasecmp(ext, "so"))
-					continue;
+		input_plugin_info = (input_plugin_info_type)
+				dlsym(handle, "input_plugin_info");
 
-				if ((handle = dlopen(path, RTLD_NOW |RTLD_GLOBAL))) {
-					input_plugin_info = (input_plugin_info_type)
-						dlsym(handle, "input_plugin_info");
+		if (!input_plugin_info) {
+			alsaplayer_error("Could not find input_plugin_info symbol in shared object `%s'", path);
+			dlclose(handle);
+			continue;
+		}
 
-					if (input_plugin_info) {
-#ifdef DEBUG
-						alsaplayer_error("Loading input plugin: %s", path);
-#endif
-						input_plugin *the_plugin = input_plugin_info();
-						if (the_plugin) {
-							the_plugin->handle = handle;
-						}
-						if (!RegisterPlugin(the_plugin)) {
-							alsaplayer_error("Error loading %s", path);
-							dlclose(handle);
-						}
-					} else {
-						alsaplayer_error("Could not find input_plugin_info symbol in shared object `%s'", path);
-						dlclose(handle);
-					}	
-				} else {
-					alsaplayer_error("%s", dlerror());
-				}
-			}	
+		//alsaplayer_error("Loading input plugin: %s", path);
+		input_plugin *the_plugin = input_plugin_info();
+		if (the_plugin) {
+			the_plugin->handle = handle;
+		}
+		if (!RegisterPlugin(the_plugin)) {
+			alsaplayer_error("Error loading %s", path);
+			dlclose(handle);
+			continue;
 		}
-		closedir(dir);
 	}
+	closedir(dir);
 }
 
 
 CorePlayer::CorePlayer(AlsaNode *the_node)
 {
 	int i;
-	file_path[0] = 0;
+
+	/* Init mutexes */
+	pthread_mutex_init(&counter_mutex, NULL); 
+	pthread_mutex_init(&player_mutex, NULL);
+	pthread_mutex_init(&notifier_mutex, NULL);
+	
 	producer_thread = 0; 
 	total_frames = 0;
 	streaming = false;
 	producing = false;
-	plugin_count = 0;
+	jumped = false;
+	//plugin_count = 0;
 	plugin = NULL;
-	jumped = jump_point = repitched = write_buf_changed = 0; 
+	jump_point = repitched = write_buf_changed = 0; 
 	new_frame_number = 0;
 	read_direction = DIR_FORWARD;
 	node = the_node;
 	pitch = 1.0;
 	sub = NULL;
 	last_read = -1;
+	input_rate = output_rate = 44100;
 	SetSpeedMulti(1.0);
 	SetSpeed(1.0);
 	SetVolume(100);
@@ -173,6 +229,9 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	buffer = NULL;
 	the_object = NULL;
 
+	save_speed = 0.0;
+
+	// allocate ringbuffer partition pointer table
 	if ((buffer = new sample_buf[NR_BUF]) == NULL) {
 		alsaplayer_error("Out of memory in CorePlayer::CorePlayer");
 		exit(1);
@@ -180,6 +239,10 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 
 	for (i=0; i < NR_BUF; i++) {
 		buffer[i].buf = new SampleBuffer(SAMPLE_STEREO, BUF_SIZE);
+		if (buffer[i].buf == NULL) {
+			alsaplayer_error("Out of memory in CorePlayer::CorePlayer");
+			exit(1);
+		}
 		// Set up next/prev pointers
 		if (i > 0) {
 			buffer[i].prev = &buffer[i-1];
@@ -191,23 +254,27 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	buffer[0].prev = &buffer[NR_BUF-1];
 	buffer[NR_BUF-1].next = &buffer[0];
 
-	memset(plugins, 0, sizeof(plugins));
+	//memset(plugins, 0, sizeof(plugins));
 
 	read_buf = write_buf = buffer;
-	pthread_mutex_init(&counter_mutex, NULL); 
-	pthread_mutex_init(&player_mutex, NULL);
 
 	the_object = new input_object;
-#ifndef NEW_PLAY	
-	the_object->write_buffer = NULL;
-#endif
 	the_object->ready = 0;
 
 	pthread_mutex_init(&the_object->object_mutex, NULL);
 	strcpy(addon_dir, ADDON_DIR);
 
 	// Load the input addons
-	load_input_addons();
+	pthread_mutex_lock(&plugins_mutex);
+	if (!plugins_loaded) {
+		plugins_loaded = 1;
+		load_input_addons();
+	}
+	if ((input_buffer = new char[128 * 1024]) == NULL) {
+		alsaplayer_error("cannot allocate mix buffer");
+		exit(1);
+	}	
+	pthread_mutex_unlock(&plugins_mutex);
 }
 
 
@@ -223,7 +290,6 @@ void CorePlayer::ResetBuffer()
 
 CorePlayer::~CorePlayer()
 {
-	input_plugin *tmp;
 	int i;
 
 	UnregisterPlugins();
@@ -236,21 +302,23 @@ CorePlayer::~CorePlayer()
 	pthread_mutex_destroy(&the_object->object_mutex);
 	delete []buffer;
 	delete the_object;
+	delete []input_buffer;
 }
 
 
 void CorePlayer::UnregisterPlugins()
 {
 	input_plugin *tmp;
-	
-	Stop();
 
+	Stop();
+	Close();
+	
 	Lock();
 	for (int i = 0; i < plugin_count; i++) {
 		tmp = &plugins[i];
 		//alsaplayer_error("Unloading Input plugin: %s", tmp->name); 
 		if (tmp->handle) {
-			tmp->shutdown(); // Shutdown plugin
+			//tmp->shutdown(); // Shutdown plugin
 			dlclose(tmp->handle);
 			tmp->handle = NULL;
 			tmp = NULL;
@@ -262,22 +330,21 @@ void CorePlayer::UnregisterPlugins()
 int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 {
 	int version;
+	int error_count = 0;
 	input_plugin *tmp;
-	
-	Lock();
+
 	tmp = &plugins[plugin_count];
 	tmp->version = the_plugin->version;
 	if (tmp->version) {
 		if ((version = tmp->version) != INPUT_PLUGIN_VERSION) {
-			alsaplayer_error("Wrong version number on plugin v%d, wanted v%d",
+			alsaplayer_error("Wrong version number on plugin (v%d, wanted v%d)",
 					version - INPUT_PLUGIN_BASE_VERSION,
 					INPUT_PLUGIN_VERSION - INPUT_PLUGIN_BASE_VERSION);      
-			Unlock();
 			return 0;
 		}
 	}
-	strncpy(tmp->name, the_plugin->name, 256);
-	strncpy(tmp->author, the_plugin->author, 256);
+	tmp->name = the_plugin->name;
+	tmp->author = the_plugin->author;
 	tmp->init = the_plugin->init;
 	tmp->can_handle = the_plugin->can_handle;
 	tmp->open = the_plugin->open;
@@ -293,19 +360,83 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 	tmp->shutdown = the_plugin->shutdown;
 	tmp->nr_tracks = the_plugin->nr_tracks;
 	tmp->track_seek = the_plugin->track_seek;
+	if (tmp->name == NULL) {
+		alsaplayer_error("No name");
+		error_count++;
+	}
+	if (tmp->author == NULL) {
+		alsaplayer_error("No author");
+		error_count++;
+	}	
+	if (tmp->init == NULL) {
+		alsaplayer_error("No init function");
+		error_count++;
+	}
+	if (tmp->can_handle == NULL) {
+		alsaplayer_error("No can_handle function");
+		error_count++;
+	}
+	if (tmp->open == NULL) {
+		alsaplayer_error("No open function");
+		error_count++;
+	}
+	if (tmp->close == NULL) {
+		alsaplayer_error("No close function");
+		error_count++;
+	}
+	if (tmp->play_frame == NULL) {
+		alsaplayer_error("No play_frame function");
+		error_count++;
+	}	
+	if (tmp->frame_seek == NULL) {
+		alsaplayer_error("No frame_seek function");
+		error_count++;
+	}
+
+	if (tmp->nr_frames == NULL) {
+		alsaplayer_error("No nr_frames function");
+		error_count++;
+	}
+	if (tmp->frame_size == NULL) {
+		alsaplayer_error("No frame_size function");
+		error_count++;
+	}
+	if (tmp->channels == NULL) {
+		alsaplayer_error("No channels function");
+		error_count++;
+	}	
+	if (tmp->stream_info == NULL) {
+		alsaplayer_error("No stream_info function");
+		error_count++;
+	}
+	if (tmp->shutdown == NULL) {
+		alsaplayer_error("No shutdown function");
+		error_count++;
+	}
+	if (error_count) {
+	    	alsaplayer_error("At least %d error(s) were detected", error_count);
+		//Unlock();
+		return 0;
+	}
+	if (!tmp->init()) {
+		alsaplayer_error("Plugin failed to initialize (\"%s\")",
+				tmp->name);
+		return 0;
+	}	
 	plugin_count++;
 	if (plugin_count == 1) { // First so assign plugin
-		plugin = tmp;
+		//plugin = tmp;
 	}
-	//alsaplayer_error("Loading Input plugin: %s", tmp->name);
-	Unlock();
+	if (global_verbose)
+		alsaplayer_error("Loading Input plugin: %s", tmp->name);
+
 	return 1;
 }
 
 
-unsigned long CorePlayer::GetCurrentTime(int frame)
+int CorePlayer::GetCurrentTime(int frame)
 {
-	unsigned long result = 0;
+	 long result = 0;
 
 	Lock();
 	if (plugin && the_object && the_object->ready) {
@@ -313,13 +444,17 @@ unsigned long CorePlayer::GetCurrentTime(int frame)
 			: frame);
 	}
 	Unlock();
-
-	return result;
+	if (result < 0) {
+		alsaplayer_error("Huh, negative????");
+	}
+	return (int)result;
 }
 
 
 int CorePlayer::GetPosition()
 {
+	int result;
+
 	if (jumped)	// HACK!
 		return jump_point;
 	if (read_buf && plugin && the_object && the_object->ready) {
@@ -327,9 +462,13 @@ int CorePlayer::GetPosition()
 			return 0;		
 
 		int frame_size = plugin->frame_size(the_object);
-		if (frame_size) 
-			return (read_buf->start + (read_buf->buf->read_index * 4) / frame_size);
-		else
+		if (frame_size)  {
+			result = (read_buf->start + (read_buf->buf->read_index * 4) / frame_size);
+			if (result < 0) {
+				alsaplayer_error("Found the error!");
+			}
+			return result;
+		} else
 			return 0;
 	} else {
 		return 0;
@@ -350,6 +489,23 @@ int CorePlayer::GetFrames()
 }
 
 
+int CorePlayer::GetTracks()
+{
+	int result = 0;
+	Lock();
+	if (plugin && the_object && the_object->ready) {
+		if (plugin->nr_tracks) {
+			result = plugin->nr_tracks(the_object);
+		} else {
+			result = 1;
+		}
+	}
+	Unlock();
+
+	return result;
+}
+
+
 int CorePlayer::GetChannels()
 {
 	return 2;	// Hardcoded for now
@@ -385,9 +541,10 @@ int CorePlayer::GetStreamInfo(stream_info *info)
 {
 	int result = 0;
 
+	memset(info, 0, sizeof(stream_info));
+	
 	Lock();
 	if (plugin && plugin->stream_info && info && the_object) {
-		memset(info, 0, sizeof(stream_info)); // Clear struct
 		result = plugin->stream_info(the_object, info);
 	}
 	Unlock();
@@ -400,42 +557,41 @@ int CorePlayer::GetStreamInfo(stream_info *info)
 // it was the most difficult to get right! We absolutely NEED to have
 // a known state of all the threads of our object! 
 
-bool CorePlayer::Start(int reset)
+bool CorePlayer::Start()
 {
 	float result;
-	int output_rate;
 	int tries;
 
+	if (IsPlaying())
+		return false;
+
 	if (!node) {
 		alsaplayer_error("CorePlayer does not have a node");
 		return false;
 	}	
-
 	Lock();
-	Close();
-	
 	// First check if we have a filename to play
-	if (!Open()) {
+	if (!the_object->ready) {
 		Unlock();
 		return false;
 	}
 	producing = true; // So Read32() doesn't generate an error
 	streaming = true;
+	jumped = false;
 
 	ResetBuffer();
 	
 	last_read = -1;
 	output_rate = node->SamplingRate();
+	input_rate = plugin->sample_rate(the_object);
 	
-	result = plugin->sample_rate(the_object);
-	if (result == output_rate)
+	if (input_rate == output_rate)
 		SetSpeedMulti(1.0);
 	else
-		SetSpeedMulti((float) result / (float) output_rate);
+		SetSpeedMulti((float) input_rate / (float) output_rate);
 	update_pitch();
 	write_buf_changed = 0;
 	read_buf = write_buf;
-
 	result = GetSpeed();
 
 	if (result < 0.0) {
@@ -446,56 +602,121 @@ bool CorePlayer::Start(int reset)
 		write_buf->start = 0;
 		SetDirection(DIR_FORWARD);
 	}
-
 	pthread_create(&producer_thread, NULL,
 		(void * (*)(void *))producer_func, this);
-
+	// allow producer to actually start producing
+	dosleep(20000);
 	//alsaplayer_error("Prebuffering...");
-	// Wait for up to 5 seconds
-	tries = 500;
-	while (--tries && !AvailableBuffers() && producing) { 
+	// wait up to 4 seconds to get all (really: half) the PCM buffers filled
+	tries = 100;
+	while (--tries && (FilledBuffers() < NR_CBUF / 2) && producing) { 
 		//alsaplayer_error("Waiting for buffers...");
-		dosleep(10000);
+		dosleep(40000);
 	}
-
 	sub = new AlsaSubscriber();
 	if (!sub) {
 		alsaplayer_error("Subscriber creation failed :-(\n");
+		Unlock();
 		return false;
-	}	
+	}
 	sub->Subscribe(node);
 	//alsaplayer_error("Starting streamer thread...");
 	sub->EnterStream(streamer_func, this);
 	Unlock();
-
+	// Notify 
+	std::set<coreplayer_notifier *>::const_iterator i;
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->start_notify) {
+			(*i)->start_notify((*i)->data);
+		}
+	}
+	UnlockNotifiers();
 	return true;
 }  
 
 
-void CorePlayer::Stop(int streamer)
+void CorePlayer::Stop()
 {
 	Lock();
+
 	if (sub) {
         	delete sub;
         	sub = NULL;
 	}
 	producing = false;
 	streaming = false;
-	pthread_mutex_unlock(&counter_mutex); // Unblock if needed
+
+	/* We don't know locked it or not */
+	pthread_mutex_trylock (&counter_mutex);
+	pthread_mutex_unlock (&counter_mutex);
+
 #ifdef DEBUG	
 	alsaplayer_error("Waiting for producer_thread to finish...");
 #endif
-	//pthread_cancel(producer_thread);
-	pthread_join(producer_thread, NULL); 
+	if (producer_thread) {
+	    if (pthread_join(producer_thread, NULL)) {
+		    alsaplayer_error("producer_thread not running...");
+	    } 
+	}	    
 	pthread_mutex_destroy(&counter_mutex);
+	
 	pthread_mutex_init(&counter_mutex, NULL);
 	producer_thread = 0;
 	
 	//alsaplayer_error("producer_func is gone now...");
 	
 	ResetBuffer();
-	Close();
 	Unlock();
+
+	// Notify 
+	std::set<coreplayer_notifier *>::const_iterator i;
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->stop_notify) {
+			(*i)->stop_notify((*i)->data);
+		}
+	}
+	UnlockNotifiers();
+}
+
+
+void CorePlayer::SetVolume(int val)
+{
+	volume = val;
+
+	std::set<coreplayer_notifier *>::const_iterator i;
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->volume_changed)
+			(*i)->volume_changed((*i)->data, volume);
+	}		
+	UnlockNotifiers();
+}
+
+void CorePlayer::SetPan(int val)
+{
+	pan = val;
+
+	std::set<coreplayer_notifier *>::const_iterator i;
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->pan_changed)
+			(*i)->pan_changed((*i)->data, pan);	
+	}
+	UnlockNotifiers();
+}
+
+
+void CorePlayer::PositionUpdate()
+{
+	std::set<coreplayer_notifier *>::const_iterator i;
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->position_notify)
+			(*i)->position_notify((*i)->data, GetPosition());
+	}
+	UnlockNotifiers();
 }
 
 
@@ -507,11 +728,23 @@ int CorePlayer::SetSpeed(float val)
 	pitch_point = val;
 	repitched = 1;
 
+	std::set<coreplayer_notifier *>::const_iterator i;
+
+	LockNotifiers();
+	for (i = notifiers.begin(); i != notifiers.end(); i++) {
+		if ((*i)->speed_changed)
+			(*i)->speed_changed((*i)->data, pitch_point);
+	}	
+	UnlockNotifiers();
+
 	return 0;
 }
 
 float CorePlayer::GetSpeed()
 {
+	if (repitched) { // Pitch was changed so return this instead
+		return pitch_point;
+	}	
 	return (read_direction == DIR_FORWARD ? pitch : -pitch);
 }
 
@@ -528,7 +761,7 @@ bool CorePlayer::CanSeek()
 int CorePlayer::Seek(int index)
 {
 	jump_point = index;
-	jumped = 1;
+	jumped = true;
 	return 0;
 }
 
@@ -545,10 +778,12 @@ int CorePlayer::FrameSeek(int index)
 
 void CorePlayer::Close()
 {
+	Lock();
 	if (plugin && the_object && the_object->ready) {
 		the_object->ready = 0;
 		plugin->close(the_object);
-	}	
+	}
+	Unlock();
 }
 
 
@@ -557,7 +792,7 @@ input_plugin *
 CorePlayer::GetPlayer(const char *path)
 {
 	// Check we've got a path
-	if (!strlen(path)) {
+	if (!*path) {
 		return NULL;
 	}
 
@@ -580,41 +815,63 @@ CorePlayer::GetPlayer(const char *path)
 	return best_plugin;
 }
 
-bool CorePlayer::Open(const char *path = NULL)
+
+bool CorePlayer::Open(const char *path)
 {
-	bool result = false;
 	input_plugin *best_plugin;
+	int frame_size = 0;
 	
-	if (path == NULL && !strlen(file_path)) {
+	Close();
+	
+	if (path == NULL || strlen(path) == 0) {
+		alsaplayer_error("No path supplied");
 		return false;
 	}
-	if (path) {
-		strncpy(file_path, path, 1023);
-		file_path[1023] = 0;
-	}	
-
-	if ((best_plugin = GetPlayer(file_path)) == NULL) {
-		alsaplayer_error("No suitable plugin found for \"%s\"", file_path);
+	if ((best_plugin = GetPlayer(path)) == NULL) {
+		alsaplayer_error("No suitable plugin found for \"%s\"", path);
 		return false;
 	}
-
-	plugin = best_plugin;
-
-	memset(the_object, 0, sizeof(the_object));
-	if (plugin->open(the_object, file_path)) {
-		result = true;
-		frames_in_buffer = read_buf->buf->GetBufferSizeBytes(plugin->frame_size(the_object)) / plugin->frame_size(the_object);
+	Lock();	
+	
+	if (best_plugin->open(the_object, path)) {
+		if ((frame_size = best_plugin->frame_size(the_object)) > BUF_SIZE) {
+			alsaplayer_error("CRITICAL ERROR: this plugin advertised a buffer size\n"
+					 "larger than %d bytes. This is not supported by AlsaPlayer!\n"
+					 "Contact the author to fix this problem.\n"
+					 "TECHNICAL: A possible solution is to divide the frame size\n"
+					 "in 2 and double the number of effective frames. This can be\n"
+					 "handled relatively easily in the plugin (hopefully).\n\n"
+					 "We will retreat, as chaos and despair await us on\n"
+					 "this chosen path........................................", BUF_SIZE);
+			best_plugin->close(the_object);
+			Unlock();
+			return false;
+		} else {
+			if (!frame_size) {
+					alsaplayer_error("No framesize");
+					best_plugin->close(the_object);
+					Unlock();
+					return false;
+			}
+			// result = true
+		}
 	} else {
-		result = false;
-		frames_in_buffer = 0;
-		plugin->close(the_object);
+		best_plugin->close(the_object);
+		Unlock();
+		return false;
 	}
-	the_object->ready = 1;
 
-	return result;	
+	the_object->ready = 1;
+	plugin = best_plugin;
+	
+	frames_in_buffer = read_buf->buf->GetBufferSizeBytes(plugin->frame_size(the_object)) / plugin->frame_size(the_object);
+
+	Unlock();
+	
+	return true;	
 }
 
-
+#ifdef DEBUG
 void print_buf(sample_buf *start)
 {
 	sample_buf *c = start;
@@ -622,12 +879,13 @@ void print_buf(sample_buf *start)
 		alsaplayer_error("%d ", c->start);
 	}
 }
+#endif
 
 int CorePlayer::SetDirection(int direction)
 {
 	sample_buf *tmp_buf;
-	
 	int buffers = 0;
+
 	if (read_direction != direction) {
 		tmp_buf = read_buf;
 		switch(direction) {
@@ -650,14 +908,15 @@ int CorePlayer::SetDirection(int direction)
 		write_buf_changed = 1;
 		read_direction = direction;
 		pthread_mutex_unlock(&counter_mutex);		
-	} else {
 	}
 	return 0;	
 	
 }
 
 
-int CorePlayer::AvailableBuffers()
+// return number of buffers already filled with (decoded) PCM data
+//
+int CorePlayer::FilledBuffers()
 {
 	int result = 0;
 	sample_buf *tmp = read_buf;
@@ -699,7 +958,9 @@ void CorePlayer::update_pitch()
 }	
 
 
-int CorePlayer::Read32(void *data, int size)
+// FIXME: some documentation needed
+//
+int CorePlayer::Read32(void *data, int samples)
 {
 	if (repitched) {
 		update_pitch();
@@ -708,9 +969,10 @@ int CorePlayer::Read32(void *data, int size)
 	if (pitch == 0.0 || (read_buf == write_buf)) {
 		if (write_buf->next->start == -2 || !producing) {
 			return -2;
-		}	
-		memset(data, 0, size * 4);
-		return size;
+		}
+		//alsaplayer_error("Nothing to play (spitting out silence)");
+		memset(data, 0, samples * 4);
+		return samples;
 	}
 	int use_read_direction = read_direction;	
 	int *out_buf = (int *)data;
@@ -724,7 +986,7 @@ int CorePlayer::Read32(void *data, int size)
 	
 	if (jumped) {
 		int i;
-		jumped = 0;
+		jumped = false;
 		new_write_buf = read_buf;
 		// ---- Testing area ----
 		sample_buf *sb;
@@ -740,7 +1002,7 @@ int CorePlayer::Read32(void *data, int size)
 	}
 	
 	if (use_read_direction == DIR_FORWARD) {
-		while (buf_index < size) {
+		while (buf_index < samples) {
 			tmp_index = (int) ((float)use_pitch*(float)(buf_index-base_corr));
 			if ((check_index + tmp_index) > (read_buf->buf->write_index)-1) {
 				if (read_buf->next->start < 0 ||
@@ -749,12 +1011,12 @@ int CorePlayer::Read32(void *data, int size)
 						//alsaplayer_error("Next in queue (2)");
 						return -2;
 					}	
-					memset(data, 0, size * 4);
+					memset(data, 0, samples * 4);
 					if (!producing) {
 						//alsaplayer_error("blah 1");
 						return -1;
 					}
-					return size;
+					return samples;
 				} else if (read_buf->next->start !=
 					read_buf->start + frames_in_buffer) {
 					alsaplayer_error("------ WTF!!? %d - %d",
@@ -773,19 +1035,19 @@ int CorePlayer::Read32(void *data, int size)
 			out_buf[buf_index++] =  *(in_buf + tmp_index);
 		}
 	} else { // DIR_BACK
-		while (buf_index < size) { // Read (size) amount of samples
+		while (buf_index < samples) {
 			tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr));
 			if ((check_index - tmp_index) < 0) { 
 				if (read_buf->prev->start < 0 ||
 					read_buf->prev == write_buf) {
-				//	printf("Back (%d %d) ", AvailableBuffers(), read_buf->prev->start);
+				//	printf("Back (%d %d) ", FilledBuffers(), read_buf->prev->start);
 				//	print_buf(read_buf->prev);
 					if (!producing) {
 						//printf("blah 2\n");
 						return -1;
 					}
-					memset(data, 0, size * 4);
-					return size;
+					memset(data, 0, samples * 4);
+					return samples;
 				}
 				read_buf = read_buf->prev;
 				pthread_mutex_unlock(&counter_mutex);
@@ -803,63 +1065,66 @@ int CorePlayer::Read32(void *data, int size)
 	}
 	read_buf->buf->Seek(check_index + ((use_read_direction == DIR_FORWARD) ?
 		 tmp_index+1 : -((tmp_index+1) > check_index ? check_index : tmp_index+1)));
-	if (!size) alsaplayer_error("Humm, I copied nothing?");
-	return size;
+	if (!samples) alsaplayer_error("Humm, I copied nothing?");
+	return samples;
 }
 
-extern int thack;
 
-int CorePlayer::pcm_worker(sample_buf *dest, int start, int len)
+int CorePlayer::pcm_worker(sample_buf *dest, int start)
 {
+	int result = 0;
 	int frames_read = 0;
 	int bytes_written = 0;
 	char *sample_buffer;
 
-	if (dest && the_object && the_object->ready) {
-		int count = dest->buf->GetBufferSizeBytes(plugin->frame_size(the_object));
-		dest->buf->Clear();
-		if (last_read == start) {
-		} else {
-			FrameSeek(start);
-		}
-		if (start < 0) {
-			return -1;
-		}
-#ifndef NEW_PLAY		
-		the_object->write_buffer = dest->buf;
-#endif		
-		sample_buffer = (char *)dest->buf->buffer_data;
-		while (count > 0 && producing) {
-			if (!plugin->play_frame(the_object, sample_buffer + bytes_written)) {
-				dest->start = start;
-#ifdef DEBUG
-				alsaplayer_error("frames read = %d", frames_read);
-#endif
-				return frames_read;
-			} else {
-				bytes_written += plugin->frame_size(the_object); // OPTIMIZE!
-				frames_read++;
-			}
-			count -= plugin->frame_size(the_object);
-		}
-		dest->start = start;
-		last_read = dest->start + frames_read;
-		dest->buf->SetSamples(bytes_written >> 2);
-		return frames_read;
-	} else {
+	if (!dest || !the_object || !the_object->ready) {
 		return -1;
-	}		
+	}
+
+	int count = dest->buf->GetBufferSizeBytes(plugin->frame_size(the_object));
+	dest->buf->Clear();
+	if (last_read != start) {
+		FrameSeek(start);
+	}
+	if (start < 0) {
+		return -1;
+	}
+	sample_buffer = (char *)dest->buf->buffer_data;
+	while (count > 0 && producing) {
+		if (!plugin->play_frame(the_object, sample_buffer + bytes_written)) {
+			dest->start = start;
+#ifdef DEBUG
+			alsaplayer_error("frames read = %d", frames_read);
+#endif
+			return frames_read;
+		} else {
+			bytes_written += plugin->frame_size(the_object); // OPTIMIZE!
+			frames_read++;
+		}
+		count -= plugin->frame_size(the_object);
+	}
+	dest->start = start;
+	last_read = dest->start + frames_read;
+	dest->buf->SetSamples(bytes_written >> 2);
+
+	// Check if rates are still the same
+	if ((result = plugin->sample_rate(the_object)) != input_rate) {
+		alsaplayer_error("WARNING: Sample rate changed in midstream (new = %d)", result);
+		input_rate = result;
+		SetSpeedMulti((float) input_rate / (float) output_rate);
+		update_pitch();
+	}	
+	return frames_read;
 }
 
 
+// fill PCM buffers with (decoded) data
+//
 void CorePlayer::producer_func(void *data)
 {
 	CorePlayer *obj = (CorePlayer *)data;
 	int frames_read;
 
-#ifndef NEW_PLAY
-	obj->the_object->write_buffer = NULL;
-#endif	
 #ifdef DEBUG
 	alsaplayer_error("Starting new producer_func...");
 #endif	
@@ -869,8 +1134,9 @@ void CorePlayer::producer_func(void *data)
 			obj->write_buf = obj->new_write_buf;
 			obj->write_buf->start = obj->new_frame_number;
 		}
-		int val = obj->AvailableBuffers();	
-		if (val < (NR_CBUF-1)) {
+		// still at least one buffer left to fill?
+		if (obj->FilledBuffers() < (NR_CBUF-1)) {
+			//alsaplayer_error("producer: filling buffer");
 			switch (obj->read_direction) {
 			 case DIR_FORWARD:
 				frames_read = obj->pcm_worker(obj->write_buf, obj->write_buf->start);
@@ -896,19 +1162,15 @@ void CorePlayer::producer_func(void *data)
 				break;
 			}
 		} else {
-#if 0
-			alsaplayer_error("producer: going to wait for a free buffer");
-#endif
+			//alsaplayer_error("producer: waiting for free buffer");
 			pthread_mutex_lock(&obj->counter_mutex);
-#if 0
-			alsaplayer_error("producer: unblocked");
-#endif
+			//alsaplayer_error("producer: unblocked");
 		}	
 	}
 	//alsaplayer_error("Exitting producer_func (producing = %d)", obj->producing);
 	pthread_exit(NULL);
-	return;
 }
+
 extern int global_bal;
 extern int global_vol_left;
 extern int global_vol_right;
@@ -922,9 +1184,8 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 {
 	CorePlayer *obj = (CorePlayer *)arg;
 	int count;
-	char input_buffer[16384]; // Should be big enough
 
-	if ((count = obj->Read32(input_buffer, size / sizeof(short))) >= 0) {
+	if ((count = obj->Read32(obj->input_buffer, size / sizeof(short))) >= 0) {
 		int v, p, left, right;
 		p = obj->pan;
 		v = obj->volume;
@@ -946,13 +1207,13 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 			}
 			//printf("v = %d, p = %d, left = %d, right = %d\n",
 			//	p, v, left, right);
-			volume_effect32(input_buffer, count, left, right);
+			volume_effect32(obj->input_buffer, count, left, right);
 		}	
 		// Now add the data to the current stream
 		int16_t *in_buffer;
 		int16_t *out_buffer;
 		int32_t level;
-		in_buffer = (int16_t *)input_buffer;
+		in_buffer = (int16_t *)obj->input_buffer;
 		out_buffer = (int16_t *)data;
 		for (int i=0; i < size ; i++) {
 				level = *(in_buffer++) + *out_buffer;
@@ -967,3 +1228,25 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 		return false;
 	}	
 }
+
+
+bool CorePlayer::IsPaused()
+{
+	return ((GetSpeed() == 0.0) ? true : false);
+}
+
+
+void CorePlayer::Pause()
+{
+	save_speed = GetSpeed ();
+	SetSpeed (0.0);
+}
+
+
+void CorePlayer::UnPause()
+{
+	if (save_speed)
+	    SetSpeed (save_speed);
+	else
+	    SetSpeed (1.0);
+}
