@@ -16,12 +16,12 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: Effects.cpp,v 1.3 2002/03/05 11:47:54 adnans Exp $
+ *  $Id: Effects.cpp,v 1.4 2003/02/16 15:06:10 adnans Exp $
  *
 */ 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include "Effects.h"
 
 static char buf[DELAY_BUF_SIZE];	
