@@ -16,17 +16,21 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <unistd.h>
 #include <pthread.h>
 #include <sys/stat.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtkdnd.h>
+#include <glib.h>
 
 #include <vector>
 #include <algorithm>
 
+#include "AlsaPlayer.h"
 #include "Playlist.h"
 #include "PlaylistWindow.h"
 #include "support.h"
@@ -35,20 +39,33 @@
 #include "utilities.h"
 #include "prefs.h"
 #include "alsaplayer_error.h"
+#include "control.h"
 
 // Forward declarations
 
 static GtkWidget *init_playlist_window(PlaylistWindowGTK *, Playlist *pl);
-static void destroy_notify(gpointer data);
 static void new_list_item(const PlayItem *item, gchar **list_item);
-static GdkColor *select_color = NULL;
-static GdkColor *default_color = NULL;
 static int current_entry = -1;
+void playlist_play_current(Playlist *playlist, GtkWidget *gtklist);
+void dialog_popup(GtkWidget *widget, gpointer data);
+
+
+// Drag & Drop stuff
+#define TARGET_URI_LIST 0x0001
+
+static GtkTargetEntry drag_types[] = {
+	        {"text/uri-list", 0, TARGET_URI_LIST}
+};
+static int n_drag_types = sizeof(drag_types)/sizeof(drag_types[0]);
+
+
 
 // Member functions
 
 PlaylistWindowGTK::PlaylistWindowGTK(Playlist * pl) {
 	playlist = pl;
+	
+	//alsaplayer_error("PlaylistWindowGTK constructor entered");
 
 	playlist_window = init_playlist_window(this, pl);
 	playlist_list = get_widget(playlist_window, "playlist");
@@ -57,8 +74,17 @@ PlaylistWindowGTK::PlaylistWindowGTK(Playlist * pl) {
 	showing = false;
 
 	pthread_mutex_init(&playlist_list_mutex, NULL);
-	//alsaplayer_error("Would Register here");
-	playlist->Register(this);
+
+	memset(&pli, 0, sizeof(playlist_interface));
+	pli.cbsetcurrent = CbSetCurrent;
+	pli.cbupdated = CbUpdated;
+	pli.cbinsert = CbInsert;
+	pli.cbremove = CbRemove;
+	pli.cbclear = CbClear;
+	pli.data = this;
+	GDK_THREADS_LEAVE();
+	playlist->Register(&pli);
+	GDK_THREADS_ENTER();
 }
 
 PlaylistWindowGTK::~PlaylistWindowGTK() {
@@ -66,9 +92,7 @@ PlaylistWindowGTK::~PlaylistWindowGTK() {
 	
 	Hide();
 	gtk_clist_clear(GTK_CLIST(playlist_list));
-	//alsaplayer_error("About to Unregister");
-	playlist->UnRegister(this);
-	//alsaplayer_error("Unregister OK");
+	playlist->UnRegister(&pli); // XXX
 }
 
 #include "pixmaps/current_play.xpm"
@@ -80,178 +104,140 @@ static GdkBitmap *current_stop_mask = (GdkBitmap *)NULL;
 
 
 // Set item currently playing
-// (FIXME - shouldn't just be the currently selected item - indicate with an
-// icon?)
-void PlaylistWindowGTK::CbSetCurrent(unsigned current) {
-	GtkWidget *working;
+void PlaylistWindowGTK::CbSetCurrent(void *data, unsigned current) {
+	PlaylistWindowGTK *gtkpl = (PlaylistWindowGTK *)data;
 	GtkStyle *style;
-#if 0	
-	if (!select_color) {
-		select_color = (GdkColor *)g_malloc(sizeof(GdkColor));
-		if (select_color) {
-			select_color->red = 200 << 8;
-			select_color->blue = 0;
-			select_color->green = 0;
-			gdk_color_alloc(gdk_colormap_get_system(), select_color);
-		}
-	}
-	if (!default_color) {
-		default_color = (GdkColor *)g_malloc(sizeof(GdkColor));
-		if (default_color) {
-			default_color->red = default_color->green = default_color->blue = 0;
-			gdk_color_alloc(gdk_colormap_get_system(), select_color);
-		}
-	}
-		
-	gtk_clist_set_foreground(GTK_CLIST(playlist_list), current_entry - 1, default_color);
-#else
+
+	GDK_THREADS_ENTER();
+	
 	if (!current_play_pix) {
-		style = gtk_widget_get_style(GTK_WIDGET(playlist_list));
-		if (!GTK_WIDGET(playlist_window)->window) {
-			gtk_widget_realize(playlist_window);
+		style = gtk_widget_get_style(GTK_WIDGET(gtkpl->playlist_list));
+		if (!GTK_WIDGET(gtkpl->playlist_window)->window) {
+			gtk_widget_realize(gtkpl->playlist_window);
 			gdk_flush();
 		}	
 		current_play_pix = gdk_pixmap_create_from_xpm_d(
-			GTK_WIDGET(playlist_window)->window,
+			GTK_WIDGET(gtkpl->playlist_window)->window,
 			&current_play_mask, &style->bg[GTK_STATE_NORMAL],
 			current_play_xpm);
 		current_stop_pix = gdk_pixmap_create_from_xpm_d(
-			GTK_WIDGET(playlist_window)->window,
+			GTK_WIDGET(gtkpl->playlist_window)->window,
 			&current_stop_mask, &style->bg[GTK_STATE_NORMAL],
 			current_stop_xpm);
 	} else {
-		gtk_clist_set_text(GTK_CLIST(playlist_list), current_entry - 1,
+		gtk_clist_set_text(GTK_CLIST(gtkpl->playlist_list), current_entry - 1,
 			0, "");
 	}	
-#endif	
 	current_entry = current;	
 	
-	//gtk_clist_select_row(GTK_CLIST(playlist_list), current_entry - 1, 1);
-#if 0	
-	gtk_clist_set_foreground(GTK_CLIST(playlist_list), current_entry - 1, select_color);	
-#else
-	gtk_clist_set_pixmap(GTK_CLIST(playlist_list), current_entry - 1,
+	gtk_clist_set_pixmap(GTK_CLIST(gtkpl->playlist_list), current_entry - 1,
 		0, current_play_pix, current_play_mask);
-#endif
-}
-
-
-void PlaylistWindowGTK::CbLock()
-{
-	GDK_THREADS_ENTER();
-	//alsaplayer_error("GDK_THREADS_ENTER()...");
-}
-
-void PlaylistWindowGTK::CbUnlock()
-{
-	//alsaplayer_error("GDK_THREADS_LEAVE()...");
+	
 	GDK_THREADS_LEAVE();
 }
 
 
-void PlaylistWindowGTK::CbUpdated(PlayItem & item, unsigned position) {
+void PlaylistWindowGTK::CbUpdated(void *data,PlayItem & item, unsigned position) {
+	PlaylistWindowGTK *gtkpl = (PlaylistWindowGTK *)data;
 	char tmp[1024];
-	int secs;
 
-	pthread_mutex_lock(&playlist_list_mutex);
+	//alsaplayer_error("About to lock list");
+	pthread_mutex_lock(&gtkpl->playlist_list_mutex);
+	//alsaplayer_error("Locked");
 
-	gtk_clist_freeze(GTK_CLIST(playlist_list));
-	//printf("Updating %d (%s, %d)\n", position, item.author.c_str(), item.playtime);
-	if (item.author.size() && item.title.size()) {
-		sprintf(tmp, "%s - %s", item.author.c_str(), item.title.c_str());
-		gtk_clist_set_text(GTK_CLIST(playlist_list), position,
+	GDK_THREADS_ENTER();
+
+	gtk_clist_freeze(GTK_CLIST(gtkpl->playlist_list));
+	if (item.title.size()) {
+		sprintf(tmp, "%s %s", item.title.c_str(),
+			item.artist.size() ? (std::string("- ") + item.artist).c_str() : "");
+		gtk_clist_set_text(GTK_CLIST(gtkpl->playlist_list), position,
 				1, g_strdup(tmp));
 	}
 	if (item.playtime >= 0) {
 		sprintf(tmp, "%02d:%02d", item.playtime / 60, item.playtime % 60);
-		gtk_clist_set_text(GTK_CLIST(playlist_list), position,
+		gtk_clist_set_text(GTK_CLIST(gtkpl->playlist_list), position,
 				2, g_strdup(tmp));
 	}	
 		
-	gtk_clist_thaw(GTK_CLIST(playlist_list));
-	pthread_mutex_unlock(&playlist_list_mutex);
+	gtk_clist_thaw(GTK_CLIST(gtkpl->playlist_list));
+	
+	GDK_THREADS_LEAVE();
+
+	pthread_mutex_unlock(&gtkpl->playlist_list_mutex);
 }
 
 // Insert new items into the displayed list
-void PlaylistWindowGTK::CbInsert(std::vector<PlayItem> & items, unsigned position) {
+void PlaylistWindowGTK::CbInsert(void *data,std::vector<PlayItem> & items, unsigned position) {
+	PlaylistWindowGTK *gtkpl = (PlaylistWindowGTK *)data;
+	
 	//alsaplayer_error("CbInsert(`%d items', %d)", items.size(), position);
 
-	pthread_mutex_lock(&playlist_list_mutex);
+	pthread_mutex_lock(&gtkpl->playlist_list_mutex);
+
+	GDK_THREADS_ENTER();
 
 	std::vector<PlayItem> item_copy = items;
 
-	GiveStatus("Adding files...");
-	gtk_clist_freeze(GTK_CLIST(playlist_list));
+	gtk_clist_freeze(GTK_CLIST(gtkpl->playlist_list));
 
 	if(items.size() > 0) {
-		int index = position;
 		std::vector<PlayItem>::const_iterator item;
 		for(item = items.begin(); item != items.end(); item++, position++) {
 			// Make a new item
 			gchar *list_item[4];
-			new_list_item(item, list_item);
+			new_list_item(&(*item), list_item);
 
 			// Add it to the playlist
-			int index = gtk_clist_insert(GTK_CLIST(playlist_list), position, list_item);
-			gtk_clist_set_shift(GTK_CLIST(playlist_list), index, 1, 2, 2);
-			gtk_clist_set_shift(GTK_CLIST(playlist_list), index, 2, 2, 2);
+			int index = gtk_clist_insert(GTK_CLIST(gtkpl->playlist_list), position, list_item);
+			gtk_clist_set_shift(GTK_CLIST(gtkpl->playlist_list), index, 1, 2, 2);
+			gtk_clist_set_shift(GTK_CLIST(gtkpl->playlist_list), index, 2, 2, 2);
 
 
 			index ++;
 		}
 	}
-    std::string msg = inttostring(items.size()) + " file";
-	if(items.size() != 1) msg += "s";
-	msg += " added";
+	gtk_clist_thaw(GTK_CLIST(gtkpl->playlist_list));
 
-	GiveStatus(msg);
-	gtk_clist_thaw(GTK_CLIST(playlist_list));
+	GDK_THREADS_LEAVE();
 
-	pthread_mutex_unlock(&playlist_list_mutex);
+	pthread_mutex_unlock(&gtkpl->playlist_list_mutex);
 }
 
 // Remove items from start to end
-void PlaylistWindowGTK::CbRemove(unsigned start, unsigned end)
+void PlaylistWindowGTK::CbRemove(void *data, unsigned start, unsigned end)
 {
+	PlaylistWindowGTK *gtkpl = (PlaylistWindowGTK *)data;
 
-	pthread_mutex_lock(&playlist_list_mutex);	
-	GiveStatus("Removing files...");
-	gtk_clist_freeze(GTK_CLIST(playlist_list));
+	pthread_mutex_lock(&gtkpl->playlist_list_mutex);	
+
+	GDK_THREADS_ENTER();
+	gtk_clist_freeze(GTK_CLIST(gtkpl->playlist_list));
 
 	unsigned i = start;
 	while(i <= end) {
-		gtk_clist_remove(GTK_CLIST(playlist_list), start - 1);
+		gtk_clist_remove(GTK_CLIST(gtkpl->playlist_list), start - 1);
 		i++;
 	}
 
-    std::string msg = inttostring(end + 1 - start) + " file";
-	if(end != start) msg += "s";
-	msg += " removed";
+	gtk_clist_thaw(GTK_CLIST(gtkpl->playlist_list));
+	GDK_THREADS_LEAVE();
 
-	GiveStatus(msg);
-
-	gtk_clist_thaw(GTK_CLIST(playlist_list));
-	
-	pthread_mutex_unlock(&playlist_list_mutex);
+	pthread_mutex_unlock(&gtkpl->playlist_list_mutex);
 }
 
 // Clear the displayed list
-void PlaylistWindowGTK::CbClear()
+void PlaylistWindowGTK::CbClear(void *data)
 {
+	PlaylistWindowGTK *gtkpl = (PlaylistWindowGTK *)data;
 
-	pthread_mutex_lock(&playlist_list_mutex);
-#ifdef DEBUG
-	printf("CbClear()\n");
-#endif /* DEBUG */
-	gtk_clist_clear(GTK_CLIST(playlist_list));
-	GiveStatus("List was cleared");
-	pthread_mutex_unlock(&playlist_list_mutex);
+	pthread_mutex_lock(&gtkpl->playlist_list_mutex);
+	GDK_THREADS_ENTER();
+	gtk_clist_clear(GTK_CLIST(gtkpl->playlist_list));
+	GDK_THREADS_LEAVE();
+	pthread_mutex_unlock(&gtkpl->playlist_list_mutex);
 }
 
-// Display a status message
-void PlaylistWindowGTK::GiveStatus(std::string status) {
-    //gtk_label_set_text(playlist_status, status.c_str());
-}
 
 // Show the playlist
 void PlaylistWindowGTK::Show() {
@@ -286,46 +272,63 @@ void PlaylistWindowGTK::ToggleVisible() {
 	}
 }
 
-// GTK Callbacks
 
 // Called when "prev" button is pressed
-extern void playlist_window_gtk_prev(GtkWidget *widget, gpointer data) {
+extern void playlist_window_gtk_prev(GtkWidget *, gpointer data) {
 	Playlist * playlist = (Playlist *) data;
 	if(playlist) {
 		playlist->Pause();
+		GDK_THREADS_LEAVE();
 		playlist->Prev();
+		GDK_THREADS_ENTER();
 		playlist->UnPause();
 	}
 }
 
+
 // Called when "next" button is pressed
-extern void playlist_window_gtk_next(GtkWidget *widget, gpointer data) {
+void playlist_window_gtk_next(GtkWidget *widget, gpointer data)
+{
 	Playlist * playlist = (Playlist *) data;
 	if(playlist) {
 		playlist->Pause();
+		GDK_THREADS_LEAVE();
 		playlist->Next();
+		GDK_THREADS_ENTER();
 		playlist->UnPause();
 	}
 }
 
+
+
 // Called when playlist is clicked to select an item
-void playlist_click(GtkWidget *widget, gint row, gint column, 
+void playlist_click(GtkWidget *widget, gint /* row */, gint /* column */, 
 					GdkEvent *bevent, gpointer data)
 {
 	Playlist *playlist = (Playlist *) data;
 	GtkWidget *win = (GtkWidget *)gtk_object_get_data(GTK_OBJECT(widget), "window");
 	if (win && (bevent && bevent->type == GDK_2BUTTON_PRESS)) {
 		// Double click - play from the clicked item
-		int selected;
-		selected = GPOINTER_TO_INT(GTK_CLIST(widget)->selection->data);
-		playlist->Pause();
-		playlist->Play(selected + 1);
-		playlist->UnPause();
+		playlist_play_current(playlist, widget);
 	}
 }
 
+
+void playlist_play_current(Playlist *playlist, GtkWidget *list)
+{
+		int selected;
+		if (!GTK_CLIST(list)->selection)
+			return;
+		selected = GPOINTER_TO_INT(GTK_CLIST(list)->selection->data);
+		GDK_THREADS_LEAVE();
+		playlist->Pause();
+		playlist->Play(selected + 1);
+		playlist->UnPause();
+		GDK_THREADS_ENTER();
+}
+
 // Called when remove button is clicked
-void playlist_remove(GtkWidget *widget, gpointer data)
+void playlist_remove(GtkWidget *, gpointer data)
 {
 	PlaylistWindowGTK *playlist_window_gtk = (PlaylistWindowGTK *) data;
 	Playlist *playlist = NULL;
@@ -349,34 +352,51 @@ void playlist_remove(GtkWidget *widget, gpointer data)
 		}	
 		while (next != start->prev) {
 			selected = GPOINTER_TO_INT(next->data);
+			GDK_THREADS_LEAVE();
 			if (playlist->GetCurrent() == selected+1) {
 				playlist->Stop();
 				playlist->Next();
+			}
+			if (playlist->Length() == (selected+1)) {
+				gtk_clist_unselect_row(GTK_CLIST(list), 
+						selected, 0);
+				//alsaplayer_error("Early trigger");
 			}	
 			playlist->Remove(selected+1, selected+1);
+			GDK_THREADS_ENTER();
 			next = next->prev;	
 		}
+		if (playlist->Length() == selected) {
+			selected--;
+		}	
+		gtk_clist_select_row(GTK_CLIST(list), selected, 0);
 	}
 }
 
 
 // Called when shuffle button clicked
-void shuffle_cb(GtkWidget *widget, gpointer data)
+void shuffle_cb(GtkWidget *, gpointer data)
 {
 	Playlist *playlist = (Playlist *)data;
-	if (playlist)
+	if (playlist) {
+		GDK_THREADS_LEAVE();
 		playlist->Shuffle();
+		GDK_THREADS_ENTER();
+	}	
 }
 
 // Called when clear button clicked
 void clear_cb(GtkWidget *widget, gpointer data)
 {
 	Playlist *playlist = (Playlist *)data;
-	if (playlist)
+	if (playlist) {
+		GDK_THREADS_LEAVE();
 		playlist->Clear();
+		GDK_THREADS_ENTER();
+	}	
 }
 
-gint list_resize(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
+gint list_resize(GtkWidget *widget, GdkEventConfigure *, gpointer data)
 {
 	GtkWidget *list = (GtkWidget *)data;	
 	GtkWidget *window;
@@ -413,7 +433,7 @@ playlist_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	return TRUE;
 }
 
-void close_cb(GtkWidget *widget, gpointer data)
+void close_cb(GtkWidget *, gpointer data)
 {
 	PlaylistWindowGTK * playlist_window_gtk = (PlaylistWindowGTK *) data;
 	if (playlist_window_gtk) {
@@ -435,7 +455,7 @@ static void new_list_item(const PlayItem *item, gchar **list_item)
 		sprintf(pt, "%02d:%02d", (item->playtime > 0) ? item->playtime / 60 : 0,
 			(item->playtime > 0) ? item->playtime % 60 : 0);
 	} else {
-		sprintf(pt, "No time");
+		sprintf(pt, "00:00");
 	}
 	list_item[2] = (gchar *)g_strdup(pt);
 	// Strip directory names
@@ -445,9 +465,10 @@ static void new_list_item(const PlayItem *item, gchar **list_item)
 			filename = (gchar *)g_strdup(dirname);
 	} else
 			filename = (gchar *)g_strdup(new_path);
-	if (item->title.size() && item->author.size()) {
-		sprintf(pt, "%s - %s", item->author.c_str(), 
-			item->title.c_str());
+	if (item->title.size()) {
+		sprintf(pt, "%s %s", item->title.c_str(), 
+			item->artist.size() ? (std::string("- ") + item->artist).c_str() :
+				"");
 	} else {
 		sprintf(pt, "%s", filename);
 	}	
@@ -457,10 +478,12 @@ static void new_list_item(const PlayItem *item, gchar **list_item)
 }
 
 
+
+
 // Called when files have been selected for adding to playlist
-void add_file_ok(GtkWidget *widget, gpointer data)
+void add_file_ok(GtkWidget *, gpointer data)
 {
-	char *selected;
+	gchar *sel;
 	PlaylistWindowGTK * playlist_window_gtk = (PlaylistWindowGTK *) data;
 	GtkWidget *add_file = playlist_window_gtk->add_file;
 
@@ -469,13 +492,8 @@ void add_file_ok(GtkWidget *widget, gpointer data)
 	GList *next = file_list->selection;
 
 	if (!playlist) {
-		printf("Invalid playlist pointer...\n");
 		return;
 	}	
-	if (!next) { // Nothing was selected
-		printf("Nothing was selected...\n");
-		return;
-	}
 	gchar *current_dir =
 		g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(add_file)));
 
@@ -485,7 +503,22 @@ void add_file_ok(GtkWidget *widget, gpointer data)
 	prefs_set_string(ap_prefs, "gtk_interface", "default_playlist_add_path", current_dir);
 
 	std::vector<std::string> paths;
-		
+	
+	if (!next) {
+		sel = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(add_file)->selection_entry)));
+		if (sel && strlen(sel)) {
+			if (!strstr(sel, "http://"))
+				paths.push_back(std::string(current_dir) + "/" + sel);
+			else
+				paths.push_back(sel);
+			GDK_THREADS_LEAVE();
+			playlist->Insert(paths, playlist->Length());
+			GDK_THREADS_ENTER();
+			g_free(sel);
+		}
+		gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(add_file)->selection_entry), "");
+		return;
+	}	
 	while (next) { // Walk the selection list
 		char *path;
 		int index = GPOINTER_TO_INT(next->data);
@@ -502,14 +535,16 @@ void add_file_ok(GtkWidget *widget, gpointer data)
 
 	// Insert all the selected paths
 	if (playlist) {
+		GDK_THREADS_LEAVE();
 		playlist->Insert(paths, playlist->Length());
+		GDK_THREADS_ENTER();
 	} else {
 		printf("No Playlist data found\n");
 	}
 }
 
 // Called when a file has been selected to be loaded as a playlist
-void load_list_ok(GtkWidget *widget, gpointer data)
+void load_list_ok(GtkWidget *, gpointer data)
 {
   PlaylistWindowGTK *playlist_window_gtk = (PlaylistWindowGTK *)data;
 	//gtk_widget_hide(GTK_WIDGET(playlist_window_gtk->load_list));
@@ -527,20 +562,22 @@ void load_list_ok(GtkWidget *widget, gpointer data)
 
 	std::string file(gtk_file_selection_get_filename(
 			GTK_FILE_SELECTION(playlist_window_gtk->load_list)));
+	GDK_THREADS_LEAVE();
 	loaderr = playlist->Load(file, playlist->Length(), false);
+	GDK_THREADS_ENTER();
 	if(loaderr == E_PL_DUBIOUS) {
 		// FIXME - pop up a dialog and check if we really want to load
-		fprintf(stderr, "Dubious whether file is a playlist - trying anyway\n");
-
+		alsaplayer_error("Dubious whether file is a playlist - trying anyway");
+		GDK_THREADS_LEAVE();
 		loaderr = playlist->Load(file, playlist->Length(), true);
+		GDK_THREADS_ENTER();
 	}
 	if(loaderr) {
 		// FIXME - pass playlist_window to this routine
-		//playlist_window->GiveStatus("File is not a valid playlist");
 	}
 }
 
-void save_list_ok(GtkWidget *widget, gpointer data)
+void save_list_ok(GtkWidget *, gpointer data)
 {
 	PlaylistWindowGTK *playlist_window_gtk = (PlaylistWindowGTK *)data;
 	gtk_widget_hide(GTK_WIDGET(playlist_window_gtk->save_list));
@@ -556,15 +593,12 @@ void save_list_ok(GtkWidget *widget, gpointer data)
 
 	std::string file(gtk_file_selection_get_filename(
 			GTK_FILE_SELECTION(playlist_window_gtk->save_list)));
-	printf("In save_list_ok()\n");
 	enum plist_result saveerr;
 	saveerr = playlist->Save(file, PL_FORMAT_M3U);
 	if(saveerr) {
 		// FIXME - pass playlist_window to this routine
-		//playlist_window->GiveStatus("Failed to save playlist");
 	} else {
 		// FIXME - pass playlist_window to this routine
-		//playlist_window->GiveStatus("Playlist saved");
 	}
 }
 
@@ -572,7 +606,7 @@ void save_list_ok(GtkWidget *widget, gpointer data)
 // Old stuff below here - this needs to be sorted out
 //
 
-void dialog_cancel(GtkWidget *widget, gpointer data)
+void dialog_cancel(GtkWidget *, gpointer data)
 {
 	GtkWidget *dialog = (GtkWidget *)data;
 
@@ -586,8 +620,42 @@ void dialog_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
 }
 
 
+void playlist_window_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	PlaylistWindowGTK *playlist_window_gtk = (PlaylistWindowGTK *) data;
+	Playlist *playlist = NULL;
+	GtkWidget *list = NULL;
 
-void dialog_popup(GtkWidget *widget, gpointer data)
+	playlist = playlist_window_gtk->GetPlaylist();
+	list = playlist_window_gtk->GetPlaylist_list();
+	
+
+	//alsaplayer_error("Key pressed!");
+	switch(event->keyval) {
+		case GDK_Insert:
+			dialog_popup(widget, (gpointer)
+				playlist_window_gtk->add_file);
+			break;	
+		case GDK_Delete:
+			playlist_remove(widget, data);
+			break;
+		case GDK_Return:
+			playlist_play_current(playlist, list);
+			break;
+		case GDK_Right:
+			// This is a hack, but quite legal
+			ap_set_position_relative(global_session_id, 10);
+			break;
+		case GDK_Left:
+			ap_set_position_relative(global_session_id, -10);
+			break;
+		default:
+			break;
+	}		
+}
+
+
+void dialog_popup(GtkWidget *, gpointer data)
 {
         GtkWidget *dialog = (GtkWidget *)data;
         gtk_widget_show(dialog);
@@ -595,18 +663,72 @@ void dialog_popup(GtkWidget *widget, gpointer data)
 
 
 
-void key_press_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void key_press_event(GtkWidget *, GdkEvent *event, gpointer data)
 {
 	//printf("Key down\n");
 }
 
 
-void button_press_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void button_press_event(GtkWidget *widget, GdkEvent *, gpointer data)
 {
 	gtk_widget_grab_focus(widget);
 }
 
 
+gint dnd_drop_event(GtkWidget *widget,
+		GdkDragContext   * context,
+		gint              /* x */,
+		gint              /* y */,
+		GtkSelectionData *selection_data,
+		guint             info,
+		guint             * /* time */,
+		void * /* data */)
+{
+	char *uri = NULL;
+	char *filename = NULL;
+	char *p, *s, *res;
+
+	if (!selection_data)
+		return 0;
+	
+	switch(info) {
+		case TARGET_URI_LIST:
+			//alsaplayer_error("TARGET_URI_LIST drop (%d,%d)", x, y);
+			uri = (char *)malloc(strlen((const char *)selection_data->data)+1);
+			strcpy(uri, (const char *)selection_data->data);
+			filename = uri;
+			while (filename) {
+				if ((p=s=strstr(filename, "\r\n"))) {
+					*s = '\0';
+					p = s+2;
+				}	
+				if (strlen(filename)) {
+					//alsaplayer_error("Adding: %s", filename);
+					res = parse_file_uri(filename);
+					if (res) {
+						GDK_THREADS_LEAVE();
+						if (is_playlist(res)) {
+							ap_add_playlist(global_session_id, res);
+						} else {
+							ap_add_path(global_session_id, res);
+						}	
+						GDK_THREADS_ENTER();
+						parse_file_uri_free(res);
+					}	
+				}
+				filename = p;
+			}
+			free(uri);
+			break;
+		default:
+			alsaplayer_error("Unknown drop!");
+			break;
+	}               
+	return 1;
+}
+
+
+
 
 
 static GtkWidget *init_playlist_window(PlaylistWindowGTK *playlist_window_gtk, Playlist *playlist)
@@ -616,16 +738,16 @@ static GtkWidget *init_playlist_window(PlaylistWindowGTK *playlist_window_gtk, P
 	GtkWidget *list;
 	GtkWidget *list_status;
 	GtkWidget *status;
+	GtkWidget *toplevel;
 	GtkStyle *style;
 	GdkFont *bold_font;
 
 	bold_font = gdk_font_load("-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-*-*");
-	
 	if (!bold_font)
 		assert ((bold_font = gdk_fontset_load("fixed")) != NULL);
 
 	playlist_window = create_playlist_window();
-
+	toplevel = gtk_widget_get_toplevel(playlist_window);
 	list = get_widget(playlist_window, "playlist");
 	gtk_object_set_data(GTK_OBJECT(list), "window", playlist_window);
 	gtk_object_set_data(GTK_OBJECT(playlist_window), "list", list);
@@ -637,7 +759,6 @@ static GtkWidget *init_playlist_window(PlaylistWindowGTK *playlist_window_gtk, P
 	style->font = bold_font;
 	gdk_font_ref(style->font);
 	gtk_widget_set_style(GTK_WIDGET(status), style);
-	
 	gtk_widget_show(status);
 	gtk_box_pack_start(GTK_BOX(list_status), status, true, false, 1);
 	
@@ -646,7 +767,6 @@ static GtkWidget *init_playlist_window(PlaylistWindowGTK *playlist_window_gtk, P
 	style = gtk_style_copy(gtk_widget_get_style(list));
 	gtk_widget_set_style(GTK_WIDGET(list), style);	
 
-
 	gtk_clist_set_column_width(GTK_CLIST(list), 0, 16);
 	gtk_clist_set_column_max_width(GTK_CLIST(list), 0, 16);
 
@@ -664,8 +784,7 @@ static GtkWidget *init_playlist_window(PlaylistWindowGTK *playlist_window_gtk, P
 		GTK_SIGNAL_FUNC(playlist_delete_event), (gpointer)playlist_window_gtk);
 	gtk_signal_connect(GTK_OBJECT(playlist_window), "delete_event",
 		GTK_SIGNAL_FUNC(playlist_delete_event), (gpointer)playlist_window_gtk);
-
-	playlist_window_gtk->add_file = gtk_file_selection_new("Add file(s)");
+	playlist_window_gtk->add_file = gtk_file_selection_new("Add files or URLs");
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(playlist_window_gtk->add_file), prefs_get_string(ap_prefs, "gtk_interface", "default_playlist_add_path", "/"));
 
 playlist_window_gtk->load_list = gtk_file_selection_new("Load Playlist");
@@ -673,7 +792,6 @@ playlist_window_gtk->load_list = gtk_file_selection_new("Load Playlist");
 
 	playlist_window_gtk->save_list = gtk_file_selection_new("Save Playlist");
 	gtk_file_selection_set_filename(GTK_FILE_SELECTION(playlist_window_gtk->save_list), prefs_get_string(ap_prefs, "gtk_interface", "default_playlist_save_path", "/"));
-
 	GtkCList *file_list = GTK_CLIST(GTK_FILE_SELECTION(playlist_window_gtk->add_file)->file_list);
 	gtk_clist_set_selection_mode(file_list, GTK_SELECTION_EXTENDED);
 
@@ -687,7 +805,12 @@ playlist_window_gtk->load_list = gtk_file_selection_new("Load Playlist");
 		"clicked", GTK_SIGNAL_FUNC(dialog_cancel), (gpointer)playlist_window_gtk->add_file);
  	gtk_signal_connect(GTK_OBJECT(playlist_window_gtk->add_file), "delete_event",
                 GTK_SIGNAL_FUNC(dialog_delete), NULL);
-	
+
+	// Shortcut keys
+	gtk_signal_connect(GTK_OBJECT(playlist_window), "key-press-event",
+		GTK_SIGNAL_FUNC(playlist_window_keypress), 
+		(gpointer)playlist_window_gtk);	
+
 	// Modify button text of add_file dialog
 	gtk_object_set(GTK_OBJECT(
 		GTK_FILE_SELECTION(playlist_window_gtk->add_file)->ok_button),
@@ -757,6 +880,16 @@ playlist_window_gtk->load_list = gtk_file_selection_new("Load Playlist");
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 			GTK_SIGNAL_FUNC(dialog_popup), (gpointer)playlist_window_gtk->load_list);
 
+	// Setup drag & drop
+	gtk_drag_dest_set(toplevel,
+			GTK_DEST_DEFAULT_ALL,
+			drag_types,
+			n_drag_types,
+			(GdkDragAction) (GDK_ACTION_MOVE | GDK_ACTION_COPY));
+	gtk_signal_connect(GTK_OBJECT(toplevel), "drag_data_received",
+			GTK_SIGNAL_FUNC(dnd_drop_event), NULL);
+
+	
 	gtk_widget_grab_focus(GTK_WIDGET(list));
 
 	return playlist_window;
