@@ -23,7 +23,6 @@
 #include <unistd.h>
 #include <sys/types.h>
 //#define NEW_SCALE
-//#define TESTING
 //#define SUBSECOND_DISPLAY 
 
 #include <algorithm>
@@ -48,7 +47,7 @@
 #endif
 #include "pixmaps/play.xpm"
 #include "pixmaps/playlist.xpm"
-#include "pixmaps/cd.xpm"
+//#include "pixmaps/cd.xpm"
 #include "pixmaps/menu.xpm"
 
 #include "PlaylistWindow.h"
@@ -57,7 +56,7 @@
 #include "CorePlayer.h"
 #include "Playlist.h"
 #include "EffectsWindow.h"
-#include "Effects.h"
+//#include "Effects.h"
 #include "ScopesWindow.h"
 Playlist *playlist = NULL;
 
@@ -74,18 +73,21 @@ Playlist *playlist = NULL;
 #define ZERO_PITCH_TRESH 2
 
 // Global variables (get rid of these too... ;-) )
-static int global_update = 1;
+int global_update = 1;
+/* These are used to contain the size of the window manager borders around
+   our windows, and are used to show/hide windows in the same positions. */
+gint global_effects_show = 0;
+gint global_scopes_show = 0;
+
+
+gint windows_x_offset = -1;
+gint windows_y_offset = -1;
 
 static int global_draw_volume = 1;
 static GtkWidget *play_pix;
 static GdkPixmap *val_ind = NULL;
-static gint global_rb = 1;
 static PlaylistWindowGTK *playlist_window_gtk = NULL;
-
-/* These are used to contain the size of the window manager borders around
-   our windows, and are used to show/hide windows in the same positions. */
-gint windows_x_offset = -1;
-gint windows_y_offset = -1;
+static coreplayer_notifier notifier;
 
 static gint main_window_x = 150;
 static gint main_window_y = 175;
@@ -99,15 +101,10 @@ typedef struct  _update_struct {
 	GtkWidget *speed_scale;
 } update_struct;
 
-update_struct global_ustr;
+static update_struct global_ustr;
 
 // Static variables  (to be moved into a class, at some point)
 static GtkWidget *play_dialog;
-static pthread_t indicator_thread;
-
-gint global_effects_show = 0;
-gint global_scopes_show = 0;
-
 static int vol_scale[] = {
 				0,1,2,4,7,12,18,26,35,45,56,69,83,100 };
 
@@ -123,46 +120,95 @@ static int vol_scale[] = {
 
 
 gint indicator_callback(gpointer data, int locking);
+void draw_speed(float speed);
+void draw_pan(int pan);
+void draw_volume(int vol);
+void position_notify(void *data, int pos);
+void notifier_lock();
+void notifier_unlock();
 
+void speed_changed(void *, float speed)
+{
+	notifier_lock();
+	draw_speed(speed);
+	notifier_unlock();
+}
 
+void pan_changed(void *, int pan)
+{
+	notifier_lock();
+	draw_pan(pan);
+	notifier_unlock();
+}
 
-gboolean main_window_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
+void volume_changed(void *, int vol)
+{
+	notifier_lock();
+	draw_volume(vol);
+	notifier_unlock();
+}
+
+void position_notify(void *, int pos)
+{
+	notifier_lock();
+	indicator_callback(NULL, 0);
+	notifier_unlock();
+}
+
+void notifier_lock(void)
 {
-	GtkFunction f = (GtkFunction)data;
-	global_update = -1;
-	//alsaplayer_error("Going to wait for indicator_thread join");
-	gdk_flush();
-	GDK_THREADS_LEAVE(); // Drop lock so indicator_thread can finish
-	pthread_join(indicator_thread, NULL);
 	GDK_THREADS_ENTER();
-	//alsaplayer_error("About to run f()");
-	if (f) { // Oh my, a very ugly HACK indeed! But it works
-		gdk_flush();
-		GDK_THREADS_LEAVE();
-		f(NULL);
-		GDK_THREADS_ENTER();
-	}
-	//alsaplayer_error("About to delete playlist_window_gtk");
-	if (playlist_window_gtk)
-		delete playlist_window_gtk;
-	//alsaplayer_error("About to call gtk_main_quit()");
-	gtk_main_quit();
+}
+
+
+void notifier_unlock(void)
+{
+	gdk_flush();
+	GDK_THREADS_LEAVE();
+}
+
+
+void stop_notify(void *data)
+{
+	//alsaplayer_error("Song was stopped");
+}
+
+void start_notify(void *data)
+{
+	//alsaplayer_error("Song was started");
+}
+
+
+gboolean main_window_delete(GtkWidget *, GdkEvent *event, gpointer data)
+{
+	global_update = -1;
+
+	// Remove notifier
 	
+	gdk_flush();
+	
+	if (playlist_window_gtk) {
+		Playlist *playlist = playlist_window_gtk->GetPlaylist();
+		GDK_THREADS_LEAVE();
+		playlist->UnRegisterNotifier(&notifier);
+		GDK_THREADS_ENTER();
+		delete playlist_window_gtk;
+	}	
+	gtk_main_quit();
+	gdk_flush();	
 	// Never reached
 	return FALSE;
 }
 
 
-void press_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void press_event(GtkWidget *, GdkEvent *, gpointer)
 {
 	global_update = 0;
 }
 
-void draw_volume();
-
-void volume_move_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void volume_move_event(GtkWidget *, GdkEvent *, gpointer)
 {
-	draw_volume();
+	//draw_volume();
 }
 
 void draw_title(char *title)
@@ -200,6 +246,7 @@ void draw_title(char *title)
 			// Do the drawing
 			gtk_widget_draw (ustr->drawing_area, &update_rect);	
 	}
+	gdk_flush();
 }
 
 void draw_format(char *format)
@@ -241,36 +288,18 @@ void draw_format(char *format)
 }
 
 
-void draw_volume()
+void draw_volume(int vol)
 {
 	update_struct *ustr = &global_ustr;
 	GtkAdjustment *adj;
-	Playlist *pl = (Playlist *)ustr->data;
-	CorePlayer *p = pl->GetCorePlayer();
 	GdkRectangle update_rect;
 	char str[60];
-	static int old_vol = -1;
-	static int count = UPDATE_COUNT;
 
 	if (!ustr->vol_scale)
 		return;
-#ifdef NEW_SCALE	
-	adj = GTK_BSCALE(ustr->vol_scale)->adjustment;
-#else	
 	adj = GTK_RANGE(ustr->vol_scale)->adjustment;
-#endif
-	int val = (int)GTK_ADJUSTMENT(adj)->value;
-
-	if (count-- > 0 && val == old_vol)
-		return;
-	else {
-		count = UPDATE_COUNT;
-		old_vol = val;
-		//p->SetVolume(val);
-	}	
-	int idx = val;
-  idx = (idx < 0) ? 0 : ((idx > 13) ? 13 : idx);
-	val = vol_scale[idx];
+	
+	int val = vol; //(int)GTK_ADJUSTMENT(adj)->value;
 
 	val ? sprintf(str, "Volume: %d%%  ", val) : sprintf(str, "Volume: mute");
 
@@ -285,35 +314,33 @@ void draw_volume()
 			gdk_draw_string(val_ind, ustr->drawing_area->style->font,
 							ustr->drawing_area->style->white_gc, update_rect.x+6, update_rect.y+12, str);
 			gtk_widget_draw (ustr->drawing_area, &update_rect);
-	}		
+	}
+	gdk_flush();
 }
 
-void draw_balance();
 
 
-void balance_move_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void pan_move_event(GtkWidget *, GdkEvent *, gpointer)
 {
 	global_draw_volume = 0;
-	draw_balance();
+	//draw_balance();
 }
 
 
-void balance_release_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void pan_release_event(GtkWidget *, GdkEvent *, gpointer)
 {
 	global_draw_volume = 1;
 }
 
 
-void draw_balance()
+void draw_pan(int val)
 {
 	update_struct *ustr = &global_ustr;
 	GdkRectangle update_rect;
-	Playlist *pl = (Playlist *)ustr->data;
-	CorePlayer *p = pl->GetCorePlayer();
 	char str[60];
-	int pan, left, right;
+	int pan;
 
-	pan = p->GetPan();
+	pan = val;
 	if (pan < 0) {
 		sprintf(str, "Pan: left %d%%", - pan);
 	} else if (pan > 0) {
@@ -336,51 +363,33 @@ void draw_balance()
 							update_rect.x+6, update_rect.y+12,
 							str);
 			gtk_widget_draw (ustr->drawing_area, &update_rect);
-	}		
+	}
+	gdk_flush();
 }
 
-void draw_speed();
 
-void speed_move_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void speed_move_event(GtkWidget *, GdkEvent *, gpointer)
 {
-	draw_speed();
+	//draw_speed();
 }
 
-void draw_speed()
+void draw_speed(float speed)
 {
 	update_struct *ustr = &global_ustr;
 	GtkAdjustment *adj;
 	GdkRectangle update_rect;
 	char str[60];
 	int speed_val;
-	static int old_val = -20000;
-	static int count = UPDATE_COUNT;
-	static int pause_blink =  2000000;
-	static int pause_active = 1;
-#if 1
+	
 	adj = GTK_RANGE(ustr->speed_scale)->adjustment;
-#else
-	adj = GTK_BSCALE(ustr->speed_scale)->adjustment;
-#endif	
 
-	speed_val = (int)GTK_ADJUSTMENT(adj)->value;
-	if (count-- > 0 && speed_val == old_val) 
-		return;
-	count = UPDATE_COUNT;	
-	old_val = speed_val;	
+	//speed_val = (int)GTK_ADJUSTMENT(adj)->value;
+	speed_val = (int)(speed * 100.0); // We need percentages
 	if (speed_val < ZERO_PITCH_TRESH && speed_val > -ZERO_PITCH_TRESH) {
-#if 0
-		if ((pause_blink -= (UPDATE_TIMEOUT * UPDATE_COUNT)) < 0) {
-			pause_blink = 200000;
-			pause_active = 1 - pause_active;
-		}
-		sprintf(str, "Speed: %s", pause_active ? "paused" : "");
-#else
 		sprintf(str, "Speed: pause");
-#endif
 	}
 	else
-		sprintf(str, "Speed: %d%%  ", (int)GTK_ADJUSTMENT(adj)->value);
+		sprintf(str, "Speed: %d%%  ", speed_val);
 	update_rect.x = 0; 
 	update_rect.y = 0;
 	update_rect.width = 82;
@@ -399,10 +408,11 @@ void draw_speed()
 							str);
 			gtk_widget_draw (ustr->drawing_area, &update_rect);		
 	}
+	gdk_flush();
 }	
 
 
-void val_release_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void val_release_event(GtkWidget *widget, GdkEvent *, gpointer)
 {
 	update_struct *ustr = &global_ustr;
 	GdkRectangle update_rect;
@@ -423,7 +433,7 @@ void val_release_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 }
 
 
-void release_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void release_event(GtkWidget *widget, GdkEvent *, gpointer)
 {
 	GtkAdjustment *adj;
 	update_struct *ustr = &global_ustr;
@@ -436,7 +446,7 @@ void release_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 }
 
 
-void move_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+void move_event(GtkWidget *, GdkEvent *, gpointer data)
 {
 	indicator_callback(data, 0);
 }
@@ -448,8 +458,10 @@ void speed_cb(GtkWidget *widget, gpointer data)
 	float val =  GTK_ADJUSTMENT(widget)->value;
 	if (val < ZERO_PITCH_TRESH && val > -ZERO_PITCH_TRESH)
 		val = 0;
-	p->SetSpeed(  (float) val / 100.0 );
-	draw_speed();
+	GDK_THREADS_LEAVE();	
+	p->SetSpeed(  (float) val / 100.0);
+	GDK_THREADS_ENTER();
+	draw_speed(val / 100.0);
 }
 
 pthread_t smoother_thread;
@@ -505,7 +517,7 @@ void smoother(void *data)
 }
 
 
-void forward_play_cb(GtkWidget *widget, gpointer data)
+void forward_play_cb(GtkWidget *, gpointer data)
 {
 	GtkAdjustment *adj;
 	int smooth_trans;
@@ -524,7 +536,7 @@ void forward_play_cb(GtkWidget *widget, gpointer data)
 }
 
 
-void reverse_play_cb(GtkWidget *widget, gpointer data)
+void reverse_play_cb(GtkWidget *, gpointer data)
 {
 	GtkAdjustment *adj;
 	int smooth_trans;
@@ -544,10 +556,9 @@ void reverse_play_cb(GtkWidget *widget, gpointer data)
 }
 
 
-void pause_cb(GtkWidget *widget, gpointer data)
+void pause_cb(GtkWidget *, gpointer data)
 {
 	GtkAdjustment *adj;
-	float new_val, temp;
 	int smooth_trans;
 
 	adj = GTK_RANGE(data)->adjustment;
@@ -573,15 +584,18 @@ void pause_cb(GtkWidget *widget, gpointer data)
 }
 
 
-void stop_cb(GtkWidget *widget, gpointer data)
+void stop_cb(GtkWidget *, gpointer data)
 {
 	Playlist *pl = (Playlist *)data;
 	CorePlayer *p = pl->GetCorePlayer();
 
 	if (p && p->IsPlaying()) {
 		pl->Pause();
+		GDK_THREADS_LEAVE();
 		p->Stop();
-		clear_buffer();
+		p->Close();
+		GDK_THREADS_ENTER();
+		//clear_buffer();
 	}	
 }
 
@@ -596,13 +610,15 @@ void play_cb(GtkWidget *widget, gpointer data)
 		if (p->IsPlaying() || !pl->Length()) {
 			eject_cb(widget, data);
 		} else if (!p->IsPlaying() && pl->Length()) {
+			GDK_THREADS_LEAVE();
 			pl->Play(pl->GetCurrent());
+			GDK_THREADS_ENTER();
 		}	
 	}	
 }
 
 
-void eject_cb(GtkWidget *wdiget, gpointer data)
+void eject_cb(GtkWidget *, gpointer data)
 {
 	Playlist *pl = (Playlist *)data;
 	CorePlayer *p = pl->GetCorePlayer();
@@ -623,12 +639,14 @@ void volume_cb(GtkWidget *widget, gpointer data)
 	if (p) {
 		int idx = (int)adj->value;
 		idx = (idx < 0) ? 0 : ((idx > 13) ? 13 : idx);
+		GDK_THREADS_LEAVE();
 		p->SetVolume(vol_scale[idx]);
+		GDK_THREADS_ENTER();
 	}
 }
 
 
-void balance_cb(GtkWidget *widget, gpointer data)
+void pan_cb(GtkWidget *widget, gpointer data)
 {
 	GtkAdjustment *adj = (GtkAdjustment *)widget;
 	Playlist *pl = (Playlist *)data;
@@ -638,12 +656,14 @@ void balance_cb(GtkWidget *widget, gpointer data)
 	if (p) {
 		val = (int)adj->value;
 		if (val > MIN_BAL_TRESH && val < MAX_BAL_TRESH) val = BAL_CENTER;
+		GDK_THREADS_LEAVE();
 		p->SetPan(val - 100);
+		GDK_THREADS_ENTER();
 	}	
 }
 
 
-gint indicator_callback(gpointer data, int locking)
+gint indicator_callback(gpointer, int locking)
 {
 	update_struct *ustr;
 	Playlist *pl;
@@ -655,9 +675,10 @@ gint indicator_callback(gpointer data, int locking)
 	stream_info info;
 	char title_string[256];
 	char str[60];
-	long slider_val, t_min, t_sec;
-	long c_hsec, secs, c_min, c_sec;
-	long sr;
+	long slider_val=0, t_min=0, t_sec=0;
+	long c_hsec=0, secs=0, c_min=0, c_sec=0;
+	long sr=0;
+	int nr_frames=0;
 	static char old_str[60] = "";
 
 	ustr = &global_ustr;
@@ -695,6 +716,7 @@ gint indicator_callback(gpointer data, int locking)
 	if (locking)
 		GDK_THREADS_LEAVE();
 	sr = p->GetSampleRate();
+	nr_frames = p->GetFrames();
 	if (p->IsActive()) { 
 		int pos;
 		pos = global_update ? p->GetPosition() : (int) adj->value;
@@ -705,10 +727,12 @@ gint indicator_callback(gpointer data, int locking)
 		c_sec = (secs % 6000) / 100;
 #ifdef SUBSECOND_DISPLAY		
 		c_hsec = secs % 100;
-#endif		
-		secs = p->GetCurrentTime(p->GetFrames());
-		t_min = secs / 6000;
-		t_sec = (secs % 6000) / 100;
+#endif	
+		if (nr_frames >= 0) {
+			secs = p->GetCurrentTime(nr_frames);
+			t_min = secs / 6000;
+			t_sec = (secs % 6000) / 100;
+		}
 		if (locking)
 			GDK_THREADS_ENTER();
 		gtk_adjustment_set_value(adj, pos);
@@ -723,16 +747,17 @@ gint indicator_callback(gpointer data, int locking)
 		c_hsec = 0;
 		sprintf(info.title, "No stream");
 	}
-	if (t_min == 0 && t_sec == 0 && !strlen(info.status)) {
-		sprintf(str, "No status");
+	if (nr_frames < 0 || strlen(info.status)) {
+		sprintf(str, "%s", info.status);
+		if (!strlen(info.status)) {
+			alsaplayer_error("empty string");
+		}	
 	} else {
-#ifdef SUBSECOND_DISPLAY	
+#ifdef SUBSECOND_DISPLAY
 		sprintf(str, "%02ld:%02ld.%02d/%02d:%02d", c_min, c_sec, c_hsec, t_min, t_sec);
 #else
-		if (!strlen(info.status))
+		if (nr_frames >= 0) 
 			sprintf(str, "%02ld:%02ld/%02ld:%02ld", c_min, c_sec, t_min, t_sec);
-		else
-			sprintf(str, "%s", info.status);
 #endif
 	}
 	if (val_ind && strcmp(old_str, str) != 0) {
@@ -765,14 +790,21 @@ gint indicator_callback(gpointer data, int locking)
 	if (locking)
 		GDK_THREADS_ENTER();
 	draw_format(info.stream_type);
-	if (strlen(info.author)) {
-		sprintf(title_string, "%s - %s", info.author, info.title);
+	if (strlen(info.artist)) {
+		sprintf(title_string, "%s - %s", info.title, info.artist);
 		draw_title(title_string);
-	} else	
-		draw_title(info.title);
-	draw_speed();
-	if (global_draw_volume)
-		draw_volume();
+	} else if (strlen(info.title)) {
+		sprintf(title_string, "%s", info.title);
+		draw_title(title_string);
+	} else {
+		char *p = strrchr(info.path, '/');
+		if (p) {
+			p++;
+			draw_title(p);
+		} else {
+			draw_title("");
+		}	
+	}
 	update_rect.x = 0;
 	update_rect.y = 0;
 	update_rect.width = ustr->drawing_area->allocation.width;
@@ -784,28 +816,30 @@ gint indicator_callback(gpointer data, int locking)
 }
 
 
-void cd_cb(GtkWidget *widget, gpointer data)
+void cd_cb(GtkWidget *, gpointer data)
 {
 	Playlist *pl = (Playlist *)data;
 	CorePlayer *p = pl->GetCorePlayer();
 
 	if (p) {
 		pl->Pause();
-		if (p->Open("CD.cdda"))
+		GDK_THREADS_LEAVE();
+		p->Stop();
+		pl->Clear();
+		if (p->Open("CD.cdda")) {
 			p->Start();
+		}	
+		GDK_THREADS_ENTER();
 		pl->UnPause();
 	}
 }
 
 
-void exit_cb(GtkWidget *widget, gpointer data)
+void exit_cb(GtkWidget *, gpointer data)
 {
 	GtkFunction f = (GtkFunction)data;
 	global_update = -1;
 	gdk_flush();
-	GDK_THREADS_LEAVE(); // Drop thread so indicator_thread can finish
-	pthread_join(indicator_thread, NULL);
-	GDK_THREADS_ENTER();
 	if (f) { // Oh my, a very ugly HACK indeed! But it works
 		GDK_THREADS_LEAVE();
 		f(NULL);
@@ -818,7 +852,7 @@ void exit_cb(GtkWidget *widget, gpointer data)
 	GDK_THREADS_ENTER();
 }
 
-void scopes_cb(GtkWidget *widget, gpointer data)
+void scopes_cb(GtkWidget *, gpointer data)
 {
 	GtkWidget *win = (GtkWidget *)data;
 	int x, y;
@@ -837,7 +871,7 @@ void scopes_cb(GtkWidget *widget, gpointer data)
 }
 
 
-void effects_cb(GtkWidget *widget, gpointer data)
+void effects_cb(GtkWidget *, gpointer data)
 {
 	GtkWidget *win = (GtkWidget *)data;
 	int x, y;
@@ -857,31 +891,41 @@ void effects_cb(GtkWidget *widget, gpointer data)
 }
 
 
-void play_file_ok(GtkWidget *widget, gpointer data)
+void play_file_ok(GtkWidget *, gpointer data)
 {
 	Playlist *playlist = (Playlist *)data;
 	CorePlayer *p = playlist->GetCorePlayer();
 
 	if (p) {
-		char *selected;
-		GtkCList *file_list =
-		GTK_CLIST(GTK_FILE_SELECTION(play_dialog)->file_list);
+		GtkCList *file_list = GTK_CLIST(GTK_FILE_SELECTION(play_dialog)->file_list);
 		GList *next = file_list->selection;
-		if (!next) { // Nothing was selected
-			return;
-		}
-		gchar *current_dir =
-		g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(play_dialog)));
+		std::vector<std::string> paths;
+		gchar *current_dir = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(play_dialog)));
 		char *path;
 		int index;	
 		int marker = strlen(current_dir)-1;
+		
 		while (marker > 0 && current_dir[marker] != '/')
 			current_dir[marker--] = '\0';
 		// Write default_play_path
 		prefs_set_string(ap_prefs, "gtk_interface", "default_play_path", current_dir);
-		
+		if (!next) {
+			gchar *sel = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(play_dialog)->selection_entry)));
+			if (sel && strlen(sel)) {
+				if (!strstr(sel, "http://"))
+					paths.push_back(std::string(current_dir) + "/" +					sel);
+				else
+					paths.push_back(sel);
+				GDK_THREADS_LEAVE();
+				playlist->AddAndPlay(paths);
+				GDK_THREADS_ENTER();
+				gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(play_dialog)->selection_entry), "");
+				g_free(sel);
+			}	
+			return;
+		}
+	
 		// Get the selections
-		std::vector<std::string> paths;
 		while (next) {
 			index = GPOINTER_TO_INT(next->data);
 
@@ -896,7 +940,9 @@ void play_file_ok(GtkWidget *widget, gpointer data)
 		sort(paths.begin(), paths.end());
 
 		// Add selections to the queue, and start playing them
+		GDK_THREADS_LEAVE();
 		playlist->AddAndPlay(paths);
+		GDK_THREADS_ENTER();
 		playlist->UnPause();
 		
 		gtk_clist_unselect_all(file_list);
@@ -906,7 +952,7 @@ void play_file_ok(GtkWidget *widget, gpointer data)
 	gtk_widget_hide(GTK_WIDGET(play_dialog));
 }
 
-void play_file_cancel(GtkWidget *widget, gpointer data)
+void play_file_cancel(GtkWidget *, gpointer data)
 {
 	gint x,y;
 
@@ -917,7 +963,7 @@ void play_file_cancel(GtkWidget *widget, gpointer data)
 }
 
 
-gboolean play_file_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+gboolean play_file_delete_event(GtkWidget *widget, GdkEvent *, gpointer)
 {
 	gint x, y;
 
@@ -928,7 +974,7 @@ gboolean play_file_delete_event(GtkWidget *widget, GdkEvent *event, gpointer dat
 	return TRUE;
 }
 
-void playlist_cb(GtkWidget *widget, gpointer data)
+void playlist_cb(GtkWidget *, gpointer data)
 {
 	PlaylistWindowGTK *pl = (PlaylistWindowGTK *)data;
 	pl->ToggleVisible();
@@ -946,21 +992,6 @@ gint alsaplayer_button_press(GtkWidget *widget, GdkEvent *event)
 }
 
 
-void indicator_looper(void *data)
-{
-#ifdef DEBUG
-	//alsaplayer_error("THREAD-%d=indicator thread", getpid());
-#endif
-	while (global_update >= 0) {
-		if (global_update == 1) {
-			indicator_callback(data, 1);
-		}
-		dosleep(UPDATE_TIMEOUT);
-	}
-	//alsaplayer_error("Exitting indicator_looper");	
-	pthread_exit(NULL);
-}
-
 
 GtkWidget *xpm_label_box(gchar * xpm_data[], GtkWidget *to_win)
 {
@@ -987,7 +1018,7 @@ GtkWidget *xpm_label_box(gchar * xpm_data[], GtkWidget *to_win)
 }
 
 
-void on_expose_event (GtkWidget * widget, GdkEvent * event, gpointer data)
+void on_expose_event (GtkWidget * widget, GdkEvent *, gpointer data)
 {
   gint x, y;
 
@@ -1004,7 +1035,7 @@ void on_expose_event (GtkWidget * widget, GdkEvent * event, gpointer data)
     }
 }
 
-gint pixmap_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
+gint pixmap_expose(GtkWidget *widget, GdkEventExpose *event, gpointer)
 {
 	GdkPixmap *the_pixmap = val_ind;
 	gdk_draw_pixmap(widget->window,
@@ -1017,7 +1048,7 @@ gint pixmap_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data)
 }
 
 
-gint val_area_configure(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
+gint val_area_configure(GtkWidget *widget, GdkEventConfigure *, gpointer)
 {
 	if (val_ind) {
 		global_update = 0;
@@ -1039,7 +1070,7 @@ gint val_area_configure(GtkWidget *widget, GdkEventConfigure *event, gpointer da
 	return true;
 }
 
-void init_main_window(Playlist *pl, GtkFunction f)
+void init_main_window(Playlist *pl)
 {
 	GtkWidget *root_menu;
 	GtkWidget *menu_item;
@@ -1063,12 +1094,11 @@ void init_main_window(Playlist *pl, GtkFunction f)
 		"AlsaPlayer" : global_session_name);
 	gtk_window_set_wmclass(GTK_WINDOW(main_window), "AlsaPlayer", "alsaplayer");
 	gtk_widget_realize(main_window);
-
 	playlist_window_gtk = new PlaylistWindowGTK(playlist);
 
 	effects_window = init_effects_window();	
 	scopes_window = init_scopes_window();
-	play_dialog = gtk_file_selection_new("Play file");
+	play_dialog = gtk_file_selection_new("Play file or URL");
 	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (play_dialog));
 	GtkCList *file_list = GTK_CLIST(GTK_FILE_SELECTION(play_dialog)->file_list);
 
@@ -1109,8 +1139,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(stop_cb), playlist);
-	gtk_button_set_relief(GTK_BUTTON(working), global_rb ? GTK_RELIEF_NONE :
-						  GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "reverse_button");
 	pix = xpm_label_box(r_play_xpm, main_window);
@@ -1118,7 +1147,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(reverse_play_cb), speed_scale);
-	gtk_button_set_relief(GTK_BUTTON(working), global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "forward_button");
 	pix = xpm_label_box(f_play_xpm, main_window);
@@ -1126,8 +1155,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix); 
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(forward_play_cb), speed_scale);
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "pause_button");
 	pix = xpm_label_box(pause_xpm, main_window);
@@ -1135,8 +1163,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(pause_cb), speed_scale);
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "prev_button");
 	pix = xpm_label_box(prev_xpm, main_window);
@@ -1144,8 +1171,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(playlist_window_gtk_prev), playlist);
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "next_button");
 	pix = xpm_label_box(next_xpm, main_window);
@@ -1153,8 +1179,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix); 
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(playlist_window_gtk_next), playlist);
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "play_button");
 	play_pix = xpm_label_box(play_xpm, main_window);
@@ -1162,8 +1187,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), play_pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(play_cb), playlist);
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "volume_pix_frame");
 	if (working) {
@@ -1173,11 +1197,8 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	}
 	working = get_widget(main_window, "vol_scale");
 	if (working) {
-#ifdef NEW_SCALE	
-		adj = GTK_BSCALE(working)->adjustment;
-#else
+		
 		adj = GTK_RANGE(working)->adjustment;
-#endif	
 		gtk_adjustment_set_value(adj, (float)(pl->GetCorePlayer())->GetVolume());
 		gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
 							GTK_SIGNAL_FUNC(volume_cb), playlist);
@@ -1213,14 +1234,14 @@ void init_main_window(Playlist *pl, GtkFunction f)
 		adj = GTK_RANGE(working)->adjustment;
 		gtk_adjustment_set_value(adj, 100.0);
 		gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-							GTK_SIGNAL_FUNC(balance_cb), playlist);
+							GTK_SIGNAL_FUNC(pan_cb), playlist);
 		global_ustr.bal_scale = working;
 		gtk_signal_connect (GTK_OBJECT (working), "motion_notify_event",
-							GTK_SIGNAL_FUNC(balance_move_event), &global_ustr);
+							GTK_SIGNAL_FUNC(pan_move_event), &global_ustr);
 		gtk_signal_connect (GTK_OBJECT (working), "button_press_event",
-							GTK_SIGNAL_FUNC(balance_move_event), &global_ustr);
+							GTK_SIGNAL_FUNC(pan_move_event), &global_ustr);
 		gtk_signal_connect (GTK_OBJECT(working), "button_release_event",
-							GTK_SIGNAL_FUNC(balance_release_event), &global_ustr);
+							GTK_SIGNAL_FUNC(pan_release_event), &global_ustr);
 	}
 
 	working = get_widget(main_window, "playlist_button");
@@ -1229,8 +1250,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	gtk_signal_connect(GTK_OBJECT(working), "clicked",
 					   GTK_SIGNAL_FUNC(playlist_cb), playlist_window_gtk); 
-	gtk_button_set_relief(GTK_BUTTON(working), 
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working), GTK_RELIEF_NONE);
 	
 	working = get_widget(main_window, "cd_button");
 	pix = xpm_label_box(menu_xpm, main_window);
@@ -1238,8 +1258,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_container_add(GTK_CONTAINER(working), pix);
 	//gtk_signal_connect(GTK_OBJECT(working), "clicked",
 	//			GTK_SIGNAL_FUNC(cd_cb), p);
-	gtk_button_set_relief(GTK_BUTTON(working),
-						  global_rb ? GTK_RELIEF_NONE : GTK_RELIEF_NORMAL);
+	gtk_button_set_relief(GTK_BUTTON(working),GTK_RELIEF_NONE);
 
 	working = get_widget(main_window, "info_box"); 
 
@@ -1272,7 +1291,7 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
 						GTK_SIGNAL_FUNC(speed_cb), playlist);
 	#endif
-	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event", GTK_SIGNAL_FUNC(main_window_delete), (void *)f);
+	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event", GTK_SIGNAL_FUNC(main_window_delete), NULL);
 
 	// Create root menu
 	root_menu = gtk_menu_new();
@@ -1326,13 +1345,12 @@ void init_main_window(Playlist *pl, GtkFunction f)
 	menu_item = gtk_menu_item_new_with_label("Exit");
 	gtk_menu_append(GTK_MENU(root_menu), menu_item);
 	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-					   GTK_SIGNAL_FUNC(exit_cb), (void *)f);
+					   GTK_SIGNAL_FUNC(exit_cb), NULL);
 	gtk_widget_show(menu_item);
 	
 	working = get_widget(main_window, "cd_button");
 	gtk_signal_connect_object (GTK_OBJECT (working), "event",
 							   GTK_SIGNAL_FUNC(alsaplayer_button_press), GTK_OBJECT(root_menu)); // cd
-
 	gdk_flush();
 
 	//gdk_window_set_decorations(GTK_WIDGET(main_window)->window, (GdkWMDecoration)0);
@@ -1343,11 +1361,30 @@ void init_main_window(Playlist *pl, GtkFunction f)
 		playlist_window_gtk->Show();
 	}	
 
-	// start indicator thread
-	//alsaplayer_error("About to start indicator_looper()");
-	pthread_create(&indicator_thread, NULL,
-				   (void * (*)(void *))indicator_looper, playlist);
-#ifdef DEBUG
-	printf("THREAD-%d=main thread\n", getpid());
+	memset(&notifier, 0, sizeof(notifier));
+	notifier.speed_changed = speed_changed;
+	notifier.pan_changed = pan_changed;
+	notifier.volume_changed = volume_changed;
+	notifier.stop_notify = stop_notify;
+	notifier.start_notify = start_notify;
+	notifier.position_notify = position_notify;
+
+	GDK_THREADS_LEAVE();
+	playlist->RegisterNotifier(&notifier, NULL);
+	GDK_THREADS_ENTER();
+#if 0
+	// Setup geometry stuff
+	memset(&geom, 0, sizeof(geom));
+	
+	toplevel = gtk_widget_get_toplevel(main_window);
+
+	geom.min_width = 408;
+	geom.min_height = geom.max_height = 98;
+	geom.max_width = 2048;	
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(toplevel),
+		GTK_WIDGET(main_window), &geom,
+		(GdkWindowHints)(GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE));
 #endif
+	gdk_flush();
 }
